/**************************************************************

phot_stars.h (C-Munipack project)
Table of stars
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef CAT_STARS_H
#define CAT_STARS_H

#include "context.h"
#include "cmpack_console.h"

/*********************   Datatypes   *******************/

/* Table of stars */
typedef struct _CmpackCatStarTab CmpackCatStarTab;

/********************   Public functions    ***********************/

/* Create the table */
CmpackCatStarTab *cat_stars_create();

/* Free allocated memory */
void cat_stars_free(CmpackCatStarTab *tab);

/* Clear the table */
void cat_stars_clear(CmpackCatStarTab *tab);

/* Dump the table */
void cat_stars_dump(CmpackCatStarTab *tab, CmpackConsole *ctx);
	
/* Write all stars to file */
void cat_stars_write(CmpackCatStarTab *tab, FILE *to);

/* Copy table of stars */
void cat_stars_copy(CmpackCatStarTab *dsttab, const CmpackCatStarTab *srcttab);

/* Creates a new star and returns its index */
int cat_stars_add(CmpackCatStarTab *tab, CmpackCatStar *star);
	
/* Creates a new star and returns its index */
int cat_stars_find(CmpackCatStarTab *tab, int star_id);

/* Get chart size */
void cat_stars_set_size(CmpackCatStarTab *tab, int width, int height);

/* Get chart size */
int cat_stars_get_size(CmpackCatStarTab *tab, int *width, int *height);

/* Creates a new star and returns its index */
CmpackCatStar *cat_star_new(int star_id);

/* Creates a new star and returns its index */
void cat_star_set(CmpackCatStar *star, int mask, double x, double y, double mag);

/* Get number of stars */
int cat_stars_count(CmpackCatStarTab *tab);

/* Get star context by its index */
CmpackCatStar *cat_stars_get(CmpackCatStarTab *tab, int index);

#endif
