/**************************************************************

file_dlg.cpp (C-Munipack project)
The preview dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <math.h>

#include "ccdfile_dlg.h"
#include "configuration.h"
#include "showheader_dlg.h"
#include "main.h"
#include "utils.h"

#define APERTURE_DEFAULT 0

enum tMenuId
{
	MENU_FILE = 1,
	MENU_VIEW
};

enum tCommandId
{
	CMD_CLOSE = 100,
	CMD_PRINT,
	CMD_EXPORT,
	CMD_PROPERTIES,
	CMD_BADPIXELS,
	CMD_PSEUDOCOLORS,
	CMD_GRAYSCALE,
	CMD_ZOOM_IN,
	CMD_ZOOM_OUT,
	CMD_ZOOM_100
};

enum tLabelId
{
	LABEL_FILE,
	LABEL_CAPTION,
	LABEL_POSITION,
	LABEL_HEIGHT,
	LABEL_SIGNAL,
	LABEL_SKY,
	LABEL_FWHM,
	LABEL_SNR,
	LABEL_APERTURE,
	LABEL_FRAME
};

enum tFieldId
{
	FIELD_FILENAME,
	FIELD_DIRPATH,
	FIELD_MAGIC,
	FIELD_DATETIME,
	FIELD_JULDAT,
	FIELD_FILTER,
	FIELD_EXPTIME,
	FIELD_CCDTEMP,
	FIELD_OBJECT,
	FIELD_OBSERVER,
	FIELD_FRAMES
};

//------------------------  COMBO BOX STRINGS   ---------------------------------

static GtkWidget *toolbar_new(GtkOrientation orientation)
{
	GtkWidget *tbox = gtk_toolbar_new();
	gtk_toolbar_set_style(GTK_TOOLBAR(tbox), GTK_TOOLBAR_ICONS);
	gtk_toolbar_set_orientation(GTK_TOOLBAR(tbox), orientation);
	return tbox;
}

static GtkToolItem *toolbar_new_button_from_stock(GtkWidget *toolbar, const gchar *stock_id)
{
	GtkToolItem *tbtn = gtk_tool_button_new_from_stock(stock_id);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tbtn, -1);
	return tbtn;
}

static GtkToolItem *toolbar_new_widget(GtkWidget *toolbar, GtkWidget *widget)
{
	GtkToolItem *item = gtk_tool_item_new();
	gtk_container_add(GTK_CONTAINER(item), widget);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
	return item;
}

static GtkWidget *toolbar_new_entry(GtkWidget *toolbar)
{
	GtkWidget *entry = gtk_entry_new();
	gtk_entry_set_width_chars(GTK_ENTRY(entry), 10);
	gtk_entry_set_alignment(GTK_ENTRY(entry), 0.5);
	gtk_editable_set_editable(GTK_EDITABLE(entry), false);
	toolbar_new_widget(toolbar, entry);
	return entry;
}

/*static GtkToolItem *toolbar_new_separator(GtkWidget *toolbar)
{
	GtkToolItem *tbtn = gtk_separator_tool_item_new();
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tbtn, -1);
	return tbtn;
}*/

//-------------------------   MENU FOR PREVIEW WINDOW   ---------------------------

//
// Constructor
//
CCCDFileDlg::CCCDFileDlg(GtkWindow *parent):CFileDlg(parent), m_Updating(false), 
	m_PhotReady(false), m_LastPosX(-1), m_LastPosY(-1), m_InfoMode(INFO_NONE)
{
	static const GdkGeometry geom = { 320, 240 };

	static const CMenuBar::tMenuItem FileMenu[] = {
		{ CMenuBar::MB_ITEM, CMD_PROPERTIES,		"Show _Properties" },
		{ CMenuBar::MB_SEPARATOR },
		//{ CMenuBar::MB_ITEM, CMD_EXPORT,		"E_xport" },
		//{ CMenuBar::MB_ITEM, CMD_PRINT,		"P_rint" },
		//{ CMenuBar::MB_SEPARATOR },
		{ CMenuBar::MB_ITEM, CMD_CLOSE,			"_Close" },
		{ CMenuBar::MB_END }
	};

	static const CMenuBar::tMenuItem ZoomMenu[] = {
		{ CMenuBar::MB_ITEM, CMD_ZOOM_IN,	"Zoom _in" },
		{ CMenuBar::MB_ITEM, CMD_ZOOM_OUT,	"Zoom _out" },
		{ CMenuBar::MB_ITEM, CMD_ZOOM_100,	"Zoom 1:1" },
		{ CMenuBar::MB_END }
	};

	static const CMenuBar::tMenuItem ViewMenu[] = {
		{ CMenuBar::MB_CHECKBTN, CMD_PSEUDOCOLORS, "_Pseudocolors" },
		{ CMenuBar::MB_CHECKBTN, CMD_BADPIXELS,	"_Bad pixels" },
		{ CMenuBar::MB_CHECKBTN, CMD_GRAYSCALE,	"_Gray scale" },
		{ CMenuBar::MB_SEPARATOR },
		{ CMenuBar::MB_SUBMENU, 0,				"_Zoom", NULL, ZoomMenu },
		{ CMenuBar::MB_END }
	};

	static const CMenuBar::tMenu Menu[] = {
		{ "_File", MENU_FILE, FileMenu },
		{ "_View", MENU_VIEW, ViewMenu },
		{ NULL }
	};

	GtkWidget *vbox, *tbox;

	gtk_window_set_geometry_hints(GTK_WINDOW(m_pDlg), NULL, (GdkGeometry*)&geom, GDK_HINT_MIN_SIZE);
	gtk_window_set_default_size(GTK_WINDOW(m_pDlg), 400, 320);

	m_Aperture = CConfig::GetInt("PreviewDlg", "Aperture", APERTURE_DEFAULT);

	// Window layout
	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(m_pDlg), vbox);

	// Menu bar
	m_Menu.Create(Menu, false);
	m_Menu.RegisterCallback(MenuClick, this);
	gtk_box_pack_start(GTK_BOX(vbox), m_Menu.Handle(), false, false, 0);

	// Toolbar
	tbox = toolbar_new(GTK_ORIENTATION_HORIZONTAL);
	gtk_box_pack_start(GTK_BOX(vbox), tbox, false, false, 0);
	m_ZoomOut = toolbar_new_button_from_stock(tbox, GTK_STOCK_ZOOM_OUT);
	m_ZoomEntry = toolbar_new_entry(tbox);
	m_ZoomIn = toolbar_new_button_from_stock(tbox, GTK_STOCK_ZOOM_IN);

	// Chart
	m_Chart.ShowImage(true);
	m_Chart.SetZoom(CConfig::GetDbl("PreviewDlg", "ZoomFactor", 1.0));
	m_Chart.SetPseudocolors(CConfig::GetBool("PreviewDlg", "Pseudocolors"));
	m_Chart.SetBadPixels(CConfig::GetBool("PreviewDlg", "BadPixels"));
	m_Chart.EnableTracking(true);
	m_Chart.RegisterCallback(ChartCallback, this);
	gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(g_object_ref(m_Chart.Handle())), true, true, 0);

	// Info box
	gtk_box_pack_start(GTK_BOX(vbox), m_InfoBox.Handle(), FALSE, FALSE, 0);
	
	// Scale
	m_ScaleBox.SetChart(&m_Chart);
	gtk_box_pack_start(GTK_BOX(vbox), m_ScaleBox.Handle(), false, true, 0);

	// Status bar
	m_Status.Create();
	gtk_box_pack_start(GTK_BOX(vbox), m_Status.Handle(), FALSE, FALSE, 0);

	// Signals
	g_signal_connect(G_OBJECT(m_ZoomIn), "clicked", G_CALLBACK(button_clicked), this);
	g_signal_connect(G_OBJECT(m_ZoomOut), "clicked", G_CALLBACK(button_clicked), this);

	// Show the dialog
	gtk_widget_show_all(vbox);
	m_ScaleBox.Show(CConfig::GetBool("PreviewDlg", "GrayScale"));
	m_InfoBox.Show(false);
	UpdateControls();
}

bool CCCDFileDlg::LoadFile(const char *fpath)
{
	m_Chart.Clear();
	m_Chart.LoadImage(fpath);
	m_Status.Set(NULL);
	m_InfoBox.Show(false);
	UpdateControls();
	return true;
}

void CCCDFileDlg::UpdateControls(void)
{
	bool ok;
	char txt[256];
	
	// Frame menu
	m_Menu.EnableMenu(MENU_FILE, true);
	m_Menu.Enable(CMD_PROPERTIES, true);
	m_Menu.Enable(CMD_EXPORT, true);
	m_Menu.Enable(CMD_PRINT, true);
	// View menu
	m_Menu.EnableMenu(MENU_VIEW, true);
	ok = true;
	gtk_widget_set_sensitive(GTK_WIDGET(m_ZoomOut), ok);
	gtk_widget_set_sensitive(GTK_WIDGET(m_ZoomEntry), ok);
	gtk_widget_set_sensitive(GTK_WIDGET(m_ZoomIn), ok);
	m_Menu.Enable(CMD_ZOOM_IN, ok);
	m_Menu.Enable(CMD_ZOOM_OUT, ok);
	m_Menu.Enable(CMD_ZOOM_100, ok);
	m_Menu.Check(CMD_PSEUDOCOLORS, m_Chart.Pseudocolors());
	m_Menu.Check(CMD_BADPIXELS, m_Chart.BadPixels());
	m_Menu.Check(CMD_GRAYSCALE, m_ScaleBox.Visible());
	// Zoom factor
	sprintf(txt, "%.0f %%", 100.0*m_Chart.Zoom());
	gtk_entry_set_text(GTK_ENTRY(m_ZoomEntry), txt);
}

void CCCDFileDlg::MenuClick(int message, int wparam, void *lparam, void *cb_data)
{
	CCCDFileDlg *pMe = (CCCDFileDlg*)cb_data;

	switch (message)
	{
	case CMenuBar::CB_ACTIVATE:
		pMe->OnCommand(wparam);
		break;
	}
}

void CCCDFileDlg::OnCommand(int cmd_id)
{
	switch (cmd_id)
	{
	// Frame menu
	case CMD_CLOSE:
		Destroy();
		break;
	case CMD_PROPERTIES:
		ShowProperties();
		break;

	// View menu
	case CMD_PSEUDOCOLORS:
		m_Chart.SetPseudocolors(m_Menu.IsChecked(CMD_PSEUDOCOLORS));
		CConfig::SetBool("PreviewDlg", "Pseudocolors", m_Chart.Pseudocolors());
		break;
	case CMD_BADPIXELS:
		m_Chart.SetBadPixels(m_Menu.IsChecked(CMD_BADPIXELS));
		CConfig::SetBool("PreviewDlg", "BadPixels", m_Chart.BadPixels());
		break;
	case CMD_GRAYSCALE:
		m_ScaleBox.Show(m_Menu.IsChecked(CMD_GRAYSCALE));
		CConfig::SetBool("PreviewDlg", "GrayScale", m_ScaleBox.Visible());
		break;

	// Zoom menu
	case CMD_ZOOM_IN:
		m_Chart.ZoomIn();
		break;
	case CMD_ZOOM_OUT:
		m_Chart.ZoomOut();
		break;
	case CMD_ZOOM_100:
		m_Chart.SetZoom(1.0);
		break;
	}
}

void CCCDFileDlg::ShowProperties(void)
{
	CCCDFileInfoDlg dlg(GTK_WINDOW(m_pDlg));
	dlg.ShowModal(m_Path, m_Chart.ImageInfo());
}

void CCCDFileDlg::button_clicked(GtkWidget *button, CCCDFileDlg *pDlg)
{
	pDlg->OnButtonClicked(button);
}

void CCCDFileDlg::OnButtonClicked(GtkWidget *pBtn)
{
	if (pBtn==GTK_WIDGET(m_ZoomIn))
		m_Chart.ZoomIn();
	else if (pBtn==GTK_WIDGET(m_ZoomOut))
		m_Chart.ZoomOut();
}

void CCCDFileDlg::ChartCallback(int message, int wparam, void* lparam, void* cb_data)
{
	CCCDFileDlg *pMe = (CCCDFileDlg*)cb_data;

	switch (message)
	{
	case CChartBase::CB_ZOOM_CHANGE:
		pMe->OnZoomChanged();
		break;
	case CChartBase::CB_MOUSE_MOVE:
		pMe->OnMouseMoved();
		break;
	case CChartBase::CB_LEFT_CLICK:
		pMe->OnLButtonClick(((GdkEventButton*)lparam)->time);
		break;
	}
}


void CCCDFileDlg::OnZoomChanged(void)
{
	char buf[256];

	sprintf(buf, "%.0f %%", 100.0*m_Chart.Zoom());
	gtk_entry_set_text(GTK_ENTRY(m_ZoomEntry), buf);
}

void CCCDFileDlg::OnMouseMoved(void)
{
	char	buf[256];
	int		x, y;
	double	val;

	if (m_Chart.GetCursorPos(&x, &y)) {
		if (x!=m_LastPosX || y!=m_LastPosY) {
			m_LastPosX = x;
			m_LastPosY = y;
			val = m_Chart.GetValue(x, y);
			sprintf(buf, "Cursor: X = %d, Y = %d, Val = %.1f ADU", x, y, val);
			m_Status.Set(buf);
		}
	} else {
		if (m_LastPosX!=-1 || m_LastPosY!=-1) {
			m_LastPosX = m_LastPosY = -1;
			m_Status.Set(NULL);
		}
	}
}


//
// Left button click
//
void CCCDFileDlg::OnLButtonClick(guint32 timestamp)
{
	int x, y;
	double aperture;

	// Quick photometry tool
	if (m_Chart.GetCursorPos(&x, &y)) {
		aperture = GetApertureRadius(Global.pProject->Aperture());
		if (aperture>0) {
			m_PhotReady = m_Chart.Photometry(x, y, aperture, &m_PhotResult);
			UpdateInfoBox();
		}
	} else {
		if (m_PhotReady) {
			m_PhotReady = false;
			UpdateInfoBox();
		}
	}
}

void CCCDFileDlg::UpdateInfoBox(void)
{
	char buf[256];

	// Quick photometry tool
	if (m_PhotReady) {
		if (m_InfoMode != INFO_PHOT) {
			m_InfoBox.Clear();
			m_InfoBox.AddHeading(LABEL_CAPTION, 0, 0);
			m_InfoBox.AddField(LABEL_POSITION, 0, 1, "Position:");
			m_InfoBox.AddField(LABEL_HEIGHT, 0, 2, "Max. value:");
			m_InfoBox.AddField(LABEL_FWHM, 0, 3, "FWHM:");
			m_InfoBox.AddField(LABEL_APERTURE, 1, 0, "Aperture:");
			m_InfoBox.AddField(LABEL_SIGNAL, 1, 1, "Signal:");
			m_InfoBox.AddField(LABEL_SKY, 1, 2, "Sky:");
			m_InfoBox.AddField(LABEL_SNR, 1, 3, "S/N ratio:");
			m_InfoMode = INFO_PHOT;
		}
		m_InfoBox.SetHeading(LABEL_CAPTION, "Quick photometry results:", 0);
		sprintf(buf, "%.1f, %.1f", m_PhotResult.center_x, m_PhotResult.center_y);
		m_InfoBox.SetField(LABEL_POSITION, buf);
		sprintf(buf, "%.1f ADU", m_PhotResult.maxval);
		m_InfoBox.SetField(LABEL_HEIGHT, buf);
		sprintf(buf, "%.1f +- %.1f ADU", m_PhotResult.skyval, m_PhotResult.skysig);
		m_InfoBox.SetField(LABEL_SKY, buf);
		if (m_PhotResult.aperture>0) {
			sprintf(buf, "%.1f pxl", m_PhotResult.aperture);
			m_InfoBox.SetField(LABEL_APERTURE, buf);
		} else {
			m_InfoBox.SetField(LABEL_APERTURE, "------");
		}
		if (m_PhotResult.signal>=0) {
			sprintf(buf, "%.1f ADU", m_PhotResult.signal);
			m_InfoBox.SetField(LABEL_SIGNAL, buf);
		} else {
			m_InfoBox.SetField(LABEL_SIGNAL, "------");
		}
		if (m_PhotResult.fwhm>0.0) {
			sprintf(buf, "%.1f pxl", m_PhotResult.fwhm);
			m_InfoBox.SetField(LABEL_FWHM, buf);
		} else {
			m_InfoBox.SetField(LABEL_FWHM, "------");
		}
		if (m_PhotResult.snr>0.0) {
			sprintf(buf, "%.1f dB", -10.0*log10(m_PhotResult.snr));
			m_InfoBox.SetField(LABEL_SNR, buf);
		} else {
			m_InfoBox.SetField(LABEL_SNR, "------");
		}
		m_InfoBox.Show(true);
	} else {
		m_InfoBox.Show(false);
	}
}

//-------------------------   FILE PREVIEW DIALOG   --------------------------------

//
// Constructor
//
CCCDFileDlgModal::CCCDFileDlgModal(GtkWindow *pParent):CFileDlgModal(pParent)
{
	static const CMenuBar::tMenuItem FileMenu[] = {
		{ CMenuBar::MB_ITEM, CMD_PROPERTIES, "_Properties" },
		{ CMenuBar::MB_END }
	};

	static const CMenuBar::tMenu PreviewMenu[] = {
		{ "_File", MENU_FILE, FileMenu },
		{ NULL }
	};

	GtkWidget *vbox;

	m_Menu.Create(PreviewMenu, false);
	m_Menu.RegisterCallback(MenuClick, this);
	m_Chart.EnableTracking(true);
	m_Chart.ShowImage(true);
	m_Chart.ShowChart(false);

	// Window layout
	vbox = GTK_DIALOG(m_pDlg)->vbox;
	gtk_box_pack_start (GTK_BOX(vbox), m_Menu.Handle(), false, false, 0);
	gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(g_object_ref(m_Chart.Handle())), true, true, 0);
	gtk_widget_show_all(vbox);
}


//
// Menu handler
//
void CCCDFileDlgModal::MenuClick(int message, int wparam, void *lparam, void *cb_data)
{
	CCCDFileDlgModal *pMe = (CCCDFileDlgModal*)cb_data;

	switch (message)
	{
	case CMenuBar::CB_ACTIVATE:
		pMe->OnCommand(wparam);
		break;
	}
}


bool CCCDFileDlgModal::LoadFile(const char *fpath)
{
	m_Chart.Clear();
	m_Chart.LoadImage(fpath);
	return true;
}

void CCCDFileDlgModal::OnCommand(int cmd_id)
{
	switch (cmd_id)
	{
	case CMD_PROPERTIES:
		ShowProperties();
		break;
	}
}

void CCCDFileDlgModal::ShowProperties(void)
{
	CCCDFileInfoDlg dlg(GTK_WINDOW(m_pDlg));
	dlg.ShowModal(m_Path, m_Chart.ImageInfo());
}

//-------------------------   CHOOSE STARS DIALOG   --------------------------------

//
// Constructor
//
CCCDFileInfoDlg::CCCDFileInfoDlg(GtkWindow *pParent):CInfoDlg(pParent), m_Path(NULL)
{
	GtkWidget *bbox;

	AddHeading(LABEL_FILE, 0, 0, "File");
	AddField(FIELD_FILENAME, 0, 1, "File name", PANGO_ELLIPSIZE_MIDDLE);
	AddField(FIELD_DIRPATH, 0, 2, "Location", PANGO_ELLIPSIZE_MIDDLE);
	AddField(FIELD_MAGIC, 0, 3, "Format");
	AddSeparator(0, 4);
	AddHeading(LABEL_FRAME, 0, 5, "Frame");
	AddField(FIELD_DATETIME, 0, 6, "Date and time");
	AddField(FIELD_JULDAT, 0, 7, "Julian date");
	AddField(FIELD_FILTER, 0, 8, "Optical filter", PANGO_ELLIPSIZE_END);
	AddField(FIELD_EXPTIME, 0, 9, "Exposure duration");
	AddField(FIELD_CCDTEMP, 0, 10, "CCD temperature");
	AddField(FIELD_FRAMES, 0, 11, "Subframes");
	AddField(FIELD_OBJECT, 0, 12, "Object", PANGO_ELLIPSIZE_END);
	AddField(FIELD_OBSERVER, 0, 13, "Observer", PANGO_ELLIPSIZE_END);
	gtk_widget_show_all(m_Tab);

	bbox = gtk_hbutton_box_new();
	gtk_box_set_spacing(GTK_BOX(bbox), 8);
	gtk_container_set_border_width(GTK_CONTAINER(bbox), 4);
	m_HdrBtn = gtk_button_new_with_label("Show header...");
	gtk_box_pack_start(GTK_BOX(bbox), m_HdrBtn, true, true, 0);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(m_pDlg)->vbox), bbox);
	gtk_widget_show_all(bbox);

	g_signal_connect(G_OBJECT(m_HdrBtn), "clicked", G_CALLBACK(button_clicked), this);
}


//
// Destructor
//
CCCDFileInfoDlg::~CCCDFileInfoDlg()
{
	if (m_Path)
		g_free(m_Path);
}


//
// Show dialog
//
void CCCDFileInfoDlg::ShowModal(const char *path, const CImageInfo *pInfo)
{
	char buf[512], *basename, *dirpath;
	CmpackDateTime dt;

	// Update properties
	g_free(m_Path);
	m_Path = g_strdup(path);
	basename = g_path_get_basename(path);
	SetField(FIELD_FILENAME, basename);
	g_free(basename);
	dirpath = g_path_get_dirname(path);
	SetField(FIELD_DIRPATH, dirpath);
	g_free(dirpath);

	SetField(FIELD_MAGIC, pInfo->Magic());
	if (pInfo->JD()>0) {
		cmpack_decodejd(pInfo->JD(), &dt);
		sprintf(buf, "%04d-%02d-%02d %d:%02d:%02d.%03d", dt.date.year, dt.date.month, dt.date.day,
			dt.time.hour, dt.time.minute, dt.time.second, dt.time.milisecond);
		SetField(FIELD_DATETIME, buf);
		SetField(FIELD_JULDAT, pInfo->JD(), 7);
	} else {
		SetField(FIELD_DATETIME, "Not available");
		SetField(FIELD_JULDAT, "Not available");
	}
	if (pInfo->Filter())
		SetField(FIELD_FILTER, pInfo->Filter());
	else
		SetField(FIELD_FILTER, "");
	if (pInfo->ExpTime()>0)
		SetField(FIELD_EXPTIME, pInfo->ExpTime(), 3, "second(s)");
	else
		SetField(FIELD_EXPTIME, "Not available");
	if (pInfo->CCDTemp()>-999)
		SetField(FIELD_CCDTEMP, pInfo->CCDTemp(), 2, "\xC2\xB0""C");
	else
		SetField(FIELD_CCDTEMP, "Not available");
	if (pInfo->Frames()>0)
		SetField(FIELD_FRAMES, pInfo->Frames(), "frame(s)");
	else
		SetField(FIELD_FRAMES, "Not available");
	if (pInfo->Object())
		SetField(FIELD_OBJECT, pInfo->Object());
	else
		SetField(FIELD_OBJECT, "");
	if (pInfo->Observer())
		SetField(FIELD_OBSERVER, pInfo->Observer());
	else
		SetField(FIELD_OBSERVER, "");

	CInfoDlg::ShowModal();
}

void CCCDFileInfoDlg::button_clicked(GtkWidget *button, CCCDFileInfoDlg *pDlg)
{
	pDlg->OnButtonClicked(button);
}

void CCCDFileInfoDlg::OnButtonClicked(GtkWidget *pBtn)
{
	if (pBtn==GTK_WIDGET(m_HdrBtn))
		ShowHeader();
}


void CCCDFileInfoDlg::ShowHeader(void)
{
	CShowHeaderDlg dlg(GTK_WINDOW(m_pDlg));
	dlg.Execute(m_Path);
}
