/**************************************************************

chart_dlg.cpp (C-Munipack project)
The 'Plot chart' dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>

#include "chartexport_dlg.h"
#include "utils.h"
#include "configuration.h"

#define MIN_WIDTH			1
#define MIN_HEIGHT			1
#define MAX_WIDTH			8192
#define MAX_HEIGHT			8192

//-------------------------   CHOOSE STARS DIALOG   --------------------------------

CChartExportDlg::CChartExportDlg(GtkWindow *pParent):m_Updating(false)
{
	GtkWidget *tbox, *frame;
	GtkObject *adj;
	GSList *group;

	// Dialog with buttons
	m_pDlg = gtk_dialog_new_with_buttons("Save chart", GTK_WINDOW(pParent), 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR),
		GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);

	// Dialog icon
	gchar *icon = get_icon_file("export");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Image size
	frame = gtk_frame_new("Image size");
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(m_pDlg)->vbox), frame, FALSE, TRUE, 0);
	tbox = gtk_table_new(4, 6, FALSE);
	gtk_container_add(GTK_CONTAINER(frame), tbox);
	gtk_container_set_border_width(GTK_CONTAINER(tbox), 4);
	gtk_table_set_row_spacings(GTK_TABLE(tbox), 4);
	gtk_table_set_row_spacing(GTK_TABLE(tbox), 1, 8);
	gtk_table_set_col_spacings(GTK_TABLE(tbox), 4);
	gtk_table_set_col_spacing(GTK_TABLE(tbox), 0, 16);
	gtk_table_set_col_spacing(GTK_TABLE(tbox), 2, 8);

	// Relative size
	m_Relative = gtk_radio_button_new_with_label(NULL, "Set size as a percentage of original size");
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Relative, 0, 6, 0, 1);
	m_RWLabel = gtk_label_new("Width:");
	gtk_misc_set_alignment(GTK_MISC(m_RWLabel), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_RWLabel, 1, 2, 1, 2);
	adj = gtk_adjustment_new(0, 0, 1000, 1, 10, 0);
	m_RelWidth = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(m_RelWidth), true);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_RelWidth, 2, 3, 1, 2);
	gtk_widget_set_size_request(m_RelWidth, 80, -1);
	m_RHLabel = gtk_label_new("Height:");
	gtk_misc_set_alignment(GTK_MISC(m_RHLabel), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_RHLabel, 3, 4, 1, 2);
	adj = gtk_adjustment_new(0, 0, 1000, 1, 10, 0);
	m_RelHeight = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(m_RelHeight), true);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_RelHeight, 4, 5, 1, 2);
	gtk_widget_set_size_request(m_RelHeight, 80, -1);
	m_RelUnit = gtk_label_new("%");
	gtk_misc_set_alignment(GTK_MISC(m_RelUnit), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_RelUnit, 5, 6, 1, 2);

	// Absolute size
	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(m_Relative));
	m_Absolute = gtk_radio_button_new_with_label(group, "Set size of exported image");
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Absolute, 0, 6, 2, 3);
	m_AWLabel = gtk_label_new("Width:");
	gtk_misc_set_alignment(GTK_MISC(m_AWLabel), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_AWLabel, 1, 2, 3, 4);
	adj = gtk_adjustment_new(0, MIN_WIDTH, MAX_WIDTH, 1, 10, 0);
	m_AbsWidth = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(m_AbsWidth), true);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_AbsWidth, 2, 3, 3, 4);
	gtk_widget_set_size_request(m_AbsWidth, 80, -1);
	m_AHLabel = gtk_label_new("Height:");
	gtk_misc_set_alignment(GTK_MISC(m_AHLabel), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_AHLabel, 3, 4, 3, 4);
	adj = gtk_adjustment_new(0, MIN_HEIGHT, MAX_HEIGHT, 1, 10, 0);
	m_AbsHeight = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(m_AbsHeight), true);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_AbsHeight, 4, 5, 3, 4);
	gtk_widget_set_size_request(m_AbsHeight, 80, -1);
	m_AbsUnit = gtk_label_new("pixels");
	gtk_misc_set_alignment(GTK_MISC(m_AbsUnit), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_AbsUnit, 5, 6, 3, 4);

	gtk_widget_show_all(GTK_DIALOG(m_pDlg)->vbox);

	// Signals:
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);
	g_signal_connect(G_OBJECT(m_Relative), "toggled", G_CALLBACK(toggled), this);
	g_signal_connect(G_OBJECT(m_Absolute), "toggled", G_CALLBACK(toggled), this);
	g_signal_connect(G_OBJECT(m_RelWidth), "value-changed", G_CALLBACK(value_changed), this);
	g_signal_connect(G_OBJECT(m_RelHeight), "value-changed", G_CALLBACK(value_changed), this);
	g_signal_connect(G_OBJECT(m_AbsWidth), "value-changed", G_CALLBACK(value_changed), this);
	g_signal_connect(G_OBJECT(m_AbsHeight), "value-changed", G_CALLBACK(value_changed), this);
}

CChartExportDlg::~CChartExportDlg()
{
	gtk_widget_destroy(m_pDlg);
}

void CChartExportDlg::Execute(const CChart *pChart)
{
	double zoom, zoom_min, zoom_max;
	char *folder, *filename, *fpath;

	m_pChart = pChart;
	if (pChart->ChartInfo()->Width()<=0 || pChart->ChartInfo()->Height()<=0)
		return;

	m_ChartWidth = m_pChart->ChartInfo()->Width();
	m_ChartHeight = m_pChart->ChartInfo()->Height();

	// Image size dialog
	m_Updating = true;
	zoom_min = Max(MIN_WIDTH/(double)m_ChartWidth, MIN_HEIGHT/(double)m_ChartHeight);
	zoom_max = Min(MAX_WIDTH/(double)m_ChartWidth, MAX_HEIGHT/(double)m_ChartHeight);
	g_object_set((GObject*)gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(m_RelWidth)), 
		"lower", 100.0 * zoom_min, "upper", 100.0 * zoom_max, NULL);
	g_object_set((GObject*)gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(m_RelHeight)), 
		"lower", 100.0 * zoom_min, "upper", 100.0 * zoom_max, NULL);
	bool mode = CConfig::GetBool("Chart", "ExportRelative", true);
	if (mode) 
		zoom = CConfig::GetDbl("Chart", "ExportZoom", 1.0);
	else
		zoom = ((double)CConfig::GetInt("Chart", "ExportWidth", m_ChartWidth)/m_ChartWidth);
	m_ImageWidth = RoundToInt(m_ChartWidth * zoom);
	m_ImageHeight = RoundToInt(m_ChartHeight * zoom);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_RelWidth), 100.0 * zoom);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_RelHeight), 100.0 * zoom);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_AbsWidth), m_ImageWidth);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_AbsHeight), m_ImageHeight);
	m_Updating = false;
	SetMode(mode);
	if (gtk_dialog_run(GTK_DIALOG(m_pDlg)) != GTK_RESPONSE_ACCEPT)
		return;

	// File chooser dialog
	GtkWidget *m_pSaveDlg = gtk_file_chooser_dialog_new("Save chart to...", GTK_WINDOW(m_pDlg),
		GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, 
		GTK_RESPONSE_ACCEPT, NULL);
	folder = CConfig::GetStr("Output", "Folder", NULL);
	if (folder) {
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(m_pSaveDlg), folder);
		g_free(folder);
	}
	filename = CConfig::GetStr("Chart", "Name", "output.png");
	if (filename) {
		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(m_pSaveDlg), filename);
		g_free(filename);
	}
	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(m_pSaveDlg), true);
	if (gtk_dialog_run(GTK_DIALOG(m_pSaveDlg)) == GTK_RESPONSE_ACCEPT) {
		// Save chart
		fpath = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(m_pSaveDlg));
		if (fpath) {
			if (SaveChart(fpath, m_ImageWidth, m_ImageHeight)) {
				folder = g_path_get_dirname(fpath);
				CConfig::SetStr("Output", "Folder", folder);
				g_free(folder);
				filename = g_path_get_basename(fpath);
				CConfig::SetStr("Chart", "Name", filename);
				g_free(filename);
			}
			g_free(fpath);
		}
	}
	gtk_widget_destroy(m_pSaveDlg);
}

void CChartExportDlg::response_dialog(GtkWidget *pDlg, gint response_id, CChartExportDlg *pMe)
{
	if (!pMe->OnResponseDialog(response_id)) 
		g_signal_stop_emission_by_name(pDlg, "response");
}

bool CChartExportDlg::OnResponseDialog(gint response_id)
{
	bool mode;
	double zoom;

	if (response_id == GTK_RESPONSE_ACCEPT) {
		mode = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_Relative));
		if (mode) {
			// Relative size
			zoom = 0.01 * gtk_spin_button_get_value(GTK_SPIN_BUTTON(m_RelWidth));
			m_ImageWidth = RoundToInt(m_ChartWidth * zoom);
			m_ImageHeight = RoundToInt(m_ChartHeight * zoom);
		} else {
			// Absolute size
			m_ImageWidth = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(m_AbsWidth));
			m_ImageHeight = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(m_AbsHeight));
			zoom = (double)m_ImageWidth / m_ChartWidth;
		}
		if (m_ImageWidth<MIN_WIDTH || m_ImageHeight<MIN_HEIGHT) {
			ShowError(GTK_WINDOW(m_pDlg), "Invalid image size");
			return false;
		}
		if (m_ImageWidth>MAX_WIDTH || m_ImageHeight>MAX_HEIGHT) {
			ShowError(GTK_WINDOW(m_pDlg), "The image is too large.");
			return false;
		}
		CConfig::SetBool("Chart", "ExportRelative", mode);
		CConfig::SetDbl("Chart", "ExportZoom", zoom);
		CConfig::SetInt("Chart", "ExportWidth", m_ImageWidth);
	}
	return true;
}

void CChartExportDlg::value_changed(GtkSpinButton *spinbutton, CChartExportDlg *pMe)
{
	pMe->OnValueChanged(spinbutton);
}

void CChartExportDlg::OnValueChanged(GtkSpinButton *spinbutton)
{
	double zoom;

	if (!m_Updating) {
		m_Updating = true;
		if (spinbutton == GTK_SPIN_BUTTON(m_RelWidth)) {
			zoom = LimitZoom(0.01 * gtk_spin_button_get_value(spinbutton));
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_RelWidth), 100.0 * zoom);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_RelHeight), 100.0 * zoom);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_AbsWidth), RoundToInt(m_ChartWidth * zoom));
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_AbsHeight), RoundToInt(m_ChartHeight * zoom));
		} else if (spinbutton == GTK_SPIN_BUTTON(m_RelHeight)) {
			zoom = LimitZoom(0.01 * gtk_spin_button_get_value(spinbutton));
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_RelWidth), 100.0 * zoom);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_RelWidth), 100.0 * zoom);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_AbsWidth), RoundToInt(m_ChartWidth * zoom));
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_AbsHeight), RoundToInt(m_ChartHeight * zoom));
		} else if (spinbutton == GTK_SPIN_BUTTON(m_AbsWidth)) {
			zoom = LimitZoom(gtk_spin_button_get_value_as_int(spinbutton) / (double)m_ChartWidth);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_RelWidth), 100.0 * zoom);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_RelHeight), 100.0 * zoom);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_AbsWidth), RoundToInt(m_ChartWidth * zoom));
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_AbsHeight), RoundToInt(m_ChartHeight * zoom));
		} else if (spinbutton == GTK_SPIN_BUTTON(m_AbsHeight)) {
			zoom = LimitZoom(gtk_spin_button_get_value_as_int(spinbutton) / (double)m_ChartHeight);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_RelWidth), 100.0 * zoom);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_RelHeight), 100.0 * zoom);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_AbsWidth), RoundToInt(m_ChartWidth * zoom));
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_AbsHeight), RoundToInt(m_ChartHeight * zoom));
		}
		m_Updating = false;
	}
}

void CChartExportDlg::toggled(GtkToggleButton *togglebutton, CChartExportDlg *pMe)
{
	pMe->OnToggled(togglebutton);
}

void CChartExportDlg::OnToggled(GtkToggleButton *togglebutton)
{
	if (togglebutton == GTK_TOGGLE_BUTTON(m_Relative))
		SetMode(true);
	else if (togglebutton == GTK_TOGGLE_BUTTON(m_Absolute))
		SetMode(false);
}

double CChartExportDlg::LimitZoom(double zoom)
{
	int width = RoundToInt(m_ChartWidth * zoom);
	if (width < MIN_WIDTH)
		zoom = MIN_WIDTH / (double)m_ChartWidth;
	if (width > MAX_WIDTH)
		zoom = MAX_WIDTH / (double)m_ChartWidth;
	int height = RoundToInt(m_ChartHeight * zoom);
	if (height < MIN_HEIGHT)
		zoom = MIN_HEIGHT / (double)m_ChartHeight;
	if (height > MAX_HEIGHT)
		zoom = MAX_HEIGHT / (double)m_ChartHeight;
	return zoom;
}

void CChartExportDlg::SetMode(bool mode)
{
	if (!m_Updating) {
		m_Updating = true;
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_Relative), mode);
		gtk_widget_set_sensitive(m_RWLabel, mode);
		gtk_widget_set_sensitive(m_RelWidth, mode);
		gtk_widget_set_sensitive(m_RHLabel, mode);
		gtk_widget_set_sensitive(m_RelHeight, mode);
		gtk_widget_set_sensitive(m_RelUnit, mode);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_Absolute), !mode);
		gtk_widget_set_sensitive(m_AWLabel, !mode);
		gtk_widget_set_sensitive(m_AbsWidth, !mode);
		gtk_widget_set_sensitive(m_AHLabel, !mode);
		gtk_widget_set_sensitive(m_AbsHeight, !mode);
		gtk_widget_set_sensitive(m_AbsUnit, !mode);
		m_Updating = false;
	}
}


//
// Save chart to file
//
bool CChartExportDlg::SaveChart(const char *fpath, int width, int height)
{
	if (fpath && width>0 && height>0) {
		cairo_surface_t *surface = cairo_image_surface_create(CAIRO_FORMAT_RGB24, width, height);
		m_pChart->Export(surface, width, height);
		cairo_surface_write_to_png(surface, fpath);
		cairo_surface_finish(surface);
		cairo_surface_destroy(surface);
		return true;
	}
	return false;
}
