/**************************************************************

darkcorr_dlg.cpp (C-Munipack project)
The 'Dark correction' dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <glib.h>
#include <cmunipack.h>

#include "darkcorr_dlg.h"
#include "main.h"
#include "progress_dlg.h"
#include "utils.h"
#include "project.h"
#include "configuration.h"
#include "proc_classes.h"

//-------------------------   MAIN WINDOW   --------------------------------

CDarkCorrDlg::CDarkCorrDlg(GtkWindow *pParent)
{
	// Dialog with buttons
	m_pDlg = gtk_file_chooser_dialog_new("Dark correction", pParent, GTK_FILE_CHOOSER_ACTION_OPEN,
		GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_EXECUTE, GTK_RESPONSE_ACCEPT, NULL);

	// Dialog icon
	gchar *icon = get_icon_file("darkcorr");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Process selection
	m_ApplyTo.Create("Apply correction to");
	gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(m_pDlg), m_ApplyTo.Handle());

	// Preview widget
	gtk_file_chooser_set_preview_widget(GTK_FILE_CHOOSER(m_pDlg), 
		GTK_WIDGET(g_object_ref(m_Preview.Handle())));
	m_Preview.SetTypeMask(FILE_TYPE_IMAGE);
	m_Preview.SetParent(GTK_WINDOW(m_pDlg));

	// Signals:
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);
	g_signal_connect(G_OBJECT(m_pDlg), "update-preview", G_CALLBACK(update_preview), this);
	g_signal_connect(G_OBJECT(m_pDlg), "selection-changed", G_CALLBACK(selection_changed), this);
}

CDarkCorrDlg::~CDarkCorrDlg()
{
	gtk_widget_destroy(m_pDlg);
}

void CDarkCorrDlg::Execute()
{
	int res;
	char msg[256];
	char *path;
	GtkWindow *pMainWnd = Global.pMainWnd->GetHandle();

	// Select dark frame
	path = CConfig::GetStr("DarkCorr", "File", NULL);
	if (path) {
		gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(m_pDlg), path);
		m_Preview.Update(path);
		g_free(path);
	} else {
		m_Preview.Clear();
	}

	gtk_file_chooser_set_preview_widget_active(GTK_FILE_CHOOSER(m_pDlg), 
		!CConfig::GetBool("Environment", "NoPreviews"));

	if (gtk_dialog_run(GTK_DIALOG(m_pDlg))==GTK_RESPONSE_ACCEPT) {
		gtk_widget_hide(m_pDlg);
		m_DarkFile = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(m_pDlg));
		CConfig::SetStr("DarkCorr", "File", m_DarkFile);
		m_FileList = NULL;
		if (m_ApplyTo.GetState()) {
			// All files
			GtkTreeModel *pList = Global.pProject->FileList();
			if (gtk_tree_model_iter_n_children(pList, NULL)>0) 
				gtk_tree_model_foreach(pList, GtkTreeModelForeachFunc(foreach_all_files), &m_FileList);
			else
				ShowError(pMainWnd, "There are no files in the project.");
		} else {
			// Selected files
			GtkTreeSelection *pSel = Global.pMainWnd->GetSelection();
			if (gtk_tree_selection_count_selected_rows(pSel)>0) 
				gtk_tree_selection_selected_foreach(pSel, GtkTreeSelectionForeachFunc(foreach_sel_files), &m_FileList);
			else
				ShowError(pMainWnd, "There are no selected files.");
		}
		if (m_FileList) {
			CProgressDlg pDlg(Global.pMainWnd->GetHandle(), "Processing DARK correction.");
			pDlg.SetMinMax(0, g_list_length(m_FileList));
			res = pDlg.Execute(ExecuteProc, this);
			if (res!=0) {
				char *msg = cmpack_formaterror(res);
				ShowError(pMainWnd, msg);
				cmpack_free(msg);
			} else
			if (m_OutFiles==0) {
				ShowError(pMainWnd, "No file was successfully processed.");
			} else {
				if (m_OutFiles!=m_InFiles) {
					sprintf(msg, "%d file(s) was successfully processed, %d file(s) failed.", 
						m_OutFiles, m_InFiles-m_OutFiles);
					ShowWarning(pMainWnd, msg);
				} else {
					sprintf(msg, "All %d file(s) was successfully processed.", m_OutFiles);
					ShowInformation(pMainWnd, msg);
				}
			}
			g_list_foreach(m_FileList, (GFunc)gtk_tree_row_reference_free, NULL);
			g_list_free(m_FileList);
			m_FileList = NULL;
		}
		g_free(m_DarkFile);
		m_DarkFile = NULL;
		Global.pProject->Save();
		Global.pProject->UpdateAllIcons();
	}
}

gboolean CDarkCorrDlg::foreach_all_files(GtkTreeModel *model, GtkTreePath *path, 
										GtkTreeIter *iter, GList **pList)
{
	GtkTreeRowReference *rowref = gtk_tree_row_reference_new(model, path);
	*pList = g_list_append(*pList, rowref);
	return FALSE;
}

void CDarkCorrDlg::foreach_sel_files(GtkTreeModel *model, GtkTreePath *path, 
							   GtkTreeIter *iter, GList **pList)
{
	GtkTreeRowReference *rowref = gtk_tree_row_reference_new(model, path);
	*pList = g_list_append(*pList, rowref);
}

void CDarkCorrDlg::response_dialog(GtkDialog *pDlg, gint response_id, CDarkCorrDlg *pMe)
{
	if (!pMe->OnCloseQuery(response_id))
		g_signal_stop_emission_by_name(pDlg, "response");
}

bool CDarkCorrDlg::OnCloseQuery(gint response_id)
{
	if (response_id == GTK_RESPONSE_ACCEPT) {
		gchar *fname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(m_pDlg));
		if (!fname || !g_file_test(fname, G_FILE_TEST_IS_REGULAR)) {
			ShowError(GTK_WINDOW(m_pDlg), "Choose a dark correction frame");
			g_free(fname);
			return false;
		}
		gchar *f = g_locale_from_utf8(fname, -1, NULL, NULL, NULL);
		if (!cmpack_ccd_test(f)) {
			ShowError(GTK_WINDOW(m_pDlg), "The selected file is not a CCD frame.");
			g_free(f);
			g_free(fname);
			return false;
		}
		g_free(f);
		g_free(fname);
	}
	return true;
}

int CDarkCorrDlg::ExecuteProc(CProgressDlg *sender, void *userdata)
{
	return ((CDarkCorrDlg*)userdata)->ProcessFiles(sender);
}

int CDarkCorrDlg::ProcessFiles(CProgressDlg *sender)
{
	int frameid, res;
	gchar *fpath, msg[128];
	GtkTreePath *path;
	CDarkCorrProc dark;

	m_InFiles = m_OutFiles = 0;
	sender->Print("------ Dark correction ------");

	res = dark.Init(sender, m_DarkFile);
	if (res==0) {
		Global.pProject->SetOrigDarkFile(m_DarkFile);
		for (GList *node = m_FileList; node != NULL; node = node->next) {
			path = gtk_tree_row_reference_get_path((GtkTreeRowReference*)node->data);
			frameid = Global.pProject->GetFrameID(path);
			fpath = Global.pProject->GetImageFileName(path);
			sender->SetFileName(fpath);
			sprintf(msg, "Frame #%d:", frameid);
			sender->Print(msg);
			g_free(fpath);
			sender->SetProgress(m_InFiles++);
			if (dark.Execute(path)==0)
				m_OutFiles++;
			gtk_tree_path_free(path);
			if (sender->Cancelled()) 
				break;
		}
		if (sender->Cancelled())
			sprintf(msg, "Cancelled at the user's request");
		else
			sprintf(msg, "====== %d succeeded, %d failed ======", m_OutFiles, m_InFiles-m_OutFiles);
		sender->Print(msg);
	} else {
		char *aux = cmpack_formaterror(res);
		sender->Print(msg);
		cmpack_free(aux);
	}
	return res;
}

void CDarkCorrDlg::update_preview(GtkFileChooser *widget, CDarkCorrDlg *pMe)
{
	pMe->OnUpdatePreview(widget);
}

void CDarkCorrDlg::OnUpdatePreview(GtkFileChooser *widget)
{
	if (!CConfig::GetBool("Environment", "NoPreviews")) {
		gchar *file = gtk_file_chooser_get_preview_filename(widget);
		if (file) {
			m_Preview.Update(file);
			g_free(file);
		} else {
			m_Preview.Clear();
		}
	}
}

void CDarkCorrDlg::selection_changed(GtkFileChooser *widget, CDarkCorrDlg *pMe)
{
	pMe->OnSelectionChanged(widget);
}

void CDarkCorrDlg::OnSelectionChanged(GtkFileChooser *widget)
{
	UpdateControls();
}

void CDarkCorrDlg::UpdateControls(void)
{
	bool ok;

	gchar *fname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(m_pDlg));
	if (fname) {
		ok = g_file_test(fname, G_FILE_TEST_IS_REGULAR);
		g_free(fname);
	} else {
		ok = false;
	}

	GtkWidget *ok_btn = get_dialog_widget_by_response(m_pDlg, GTK_RESPONSE_ACCEPT);
	gtk_widget_set_sensitive(ok_btn, ok);
}
