/**************************************************************

file_dlg.cpp (C-Munipack project)
The preview dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>

#include "file_dlg.h"
#include "main.h"
#include "utils.h"

//-------------------------   FILE PREVIEW WINDOW   --------------------------------

//
// Constructor
//
CFileDlg::CFileDlg(GtkWindow *pParent):m_pParent(pParent), m_Path(NULL), m_FileName(NULL), 
	m_DirPath(NULL)
{
	// Dialog with buttons
	m_pDlg = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_destroy_with_parent(GTK_WINDOW(m_pDlg), true);
	gtk_window_set_position(GTK_WINDOW(m_pDlg), GTK_WIN_POS_CENTER);
	g_signal_connect(G_OBJECT(m_pDlg), "destroy", G_CALLBACK(destroy), this);
}


//
// Destructor
//
CFileDlg::~CFileDlg()
{
	if (m_Path) 
		g_free(m_Path);
	if (m_FileName) 
		g_free(m_FileName);
	if (m_DirPath) 
		g_free(m_DirPath);
}


//
// Load a file
//
bool CFileDlg::Load(const char *fpath)
{
	SetPath(fpath);
	return LoadFile(fpath);
}


//
// Show a window
//
void CFileDlg::Show()
{
	if (m_Path) {
		gtk_window_set_title(GTK_WINDOW(m_pDlg), m_FileName);
		OnBeforeShow();
		gtk_window_present(GTK_WINDOW(m_pDlg));
	}
}


//
// Hide a window
//
void CFileDlg::Destroy()
{
	gtk_widget_destroy(GTK_WIDGET(m_pDlg));
}


//
// Set file path
//
void CFileDlg::SetPath(const char *fpath)
{
	if (m_Path) {
		g_free(m_Path);
		m_Path = NULL;
	}
	if (m_FileName) {
		g_free(m_FileName);
		m_FileName = NULL;
	}
	if (m_DirPath) {
		g_free(m_DirPath);
		m_DirPath = NULL;
	}
	if (fpath) {
		m_Path = g_strdup(fpath);
		m_FileName = g_path_get_basename(fpath);
		m_DirPath = g_path_get_dirname(fpath);
	}
}

void CFileDlg::destroy(GtkObject *pWnd, CFileDlg *pDlg)
{
	if (Global.pMainWnd)
		Global.pMainWnd->FileClosed(pDlg);
	delete pDlg;
}

void CFileDlg::delete_event(GtkWidget *pWnd, GdkEvent *event, CFileDlg *pDlg)
{
	gtk_widget_destroy(pWnd);
}

//-------------------------   FILE PREVIEW DIALOG   --------------------------------

//
// Constructor
//
CFileDlgModal::CFileDlgModal(GtkWindow *pParent):m_pParent(pParent), m_Path(NULL), 
	m_FileName(NULL), m_DirPath(NULL)
{
	static const GdkGeometry geom = { 320, 240 };

	// Dialog with buttons
	m_pDlg = gtk_dialog_new_with_buttons("", pParent, 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR),
		GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
	gtk_window_set_geometry_hints(GTK_WINDOW(m_pDlg), NULL, (GdkGeometry*)&geom, GDK_HINT_MIN_SIZE);
	gtk_window_set_position(GTK_WINDOW(m_pDlg), GTK_WIN_POS_CENTER);
	gtk_window_set_default_size(GTK_WINDOW(m_pDlg), 400, 320);
}


//
// Destructor
//
CFileDlgModal::~CFileDlgModal()
{
	if (m_Path) 
		g_free(m_Path);
	if (m_FileName) 
		g_free(m_FileName);
	if (m_DirPath) 
		g_free(m_DirPath);
	gtk_widget_destroy(m_pDlg);
}


//
// Show modal dialog
//
void CFileDlgModal::ShowModal(const char *fpath)
{
	if (LoadFile(fpath)) {
		SetPath(fpath);
		gtk_window_set_title(GTK_WINDOW(m_pDlg), m_FileName);
		gtk_dialog_run(GTK_DIALOG(m_pDlg));
	}
}

//
// Set file path
//
void CFileDlgModal::SetPath(const char *fpath)
{
	if (m_Path) {
		g_free(m_Path);
		m_Path = NULL;
	}
	if (m_FileName) {
		g_free(m_FileName);
		m_FileName = NULL;
	}
	if (m_DirPath) {
		g_free(m_DirPath);
		m_DirPath = NULL;
	}
	if (fpath) {
		m_Path = g_strdup(fpath);
		m_FileName = g_path_get_basename(fpath);
		m_DirPath = g_path_get_dirname(fpath);
	}
}
