/**************************************************************

preview_dlg.cpp (C-Munipack project)
The preview dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <stdio.h>
#include <math.h>

#include "project.h"
#include "chart.h"
#include "frame_dlg.h"
#include "frameinfo_dlg.h"
#include "main.h"
#include "utils.h"
#include "configuration.h"
#include "showheader_dlg.h"

#define APERTURE_DEFAULT 0

enum tChartMode
{
	CHART_OFF,			// No chart (hide stars)
	CHART_PHOT,			// Photometry file (all stars)
	CHART_MATCH			// Matched stars only
};

enum tMenuId
{
	MENU_FRAME = 1,
	MENU_NAVIGATE,
	MENU_VIEW
};

enum tCommandId
{
	CMD_CLOSE = 100,
	CMD_PRINT,
	CMD_EXPORT,
	CMD_PROPERTIES,
	CMD_REMOVE,
	CMD_GOTO_FIRST,
	CMD_GOTO_LAST,
	CMD_GOTO_PREV,
	CMD_GOTO_NEXT,
	CMD_BADPIXELS,
	CMD_PSEUDOCOLORS,
	CMD_GRAYSCALE,
	CMD_ORIGINAL,
	CMD_IMAGE,
	CMD_CHART,
	CMD_IMAGE_CHART,
	CMD_BADSTARS,
	CMD_APERTURE,
	CMD_ZOOM_IN,
	CMD_ZOOM_OUT,
	CMD_ZOOM_100
};

enum tFieldId
{
	LABEL_CAPTION,
	LABEL_POSITION,
	LABEL_MAGNITUDE,
	LABEL_REFERENCE,
	LABEL_HEIGHT,
	LABEL_SIGNAL,
	LABEL_SKY,
	LABEL_FWHM,
	LABEL_SNR,
	LABEL_APERTURE
};

//-------------------------   MENU FOR PREVIEW WINDOW   ---------------------------

static const CMenuBar::tMenuItem FrameMenu[] = {
	{ CMenuBar::MB_ITEM, CMD_PROPERTIES,			"Show _Properties" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM, CMD_REMOVE,				"_Remove from project" },
	//{ CMenuBar::MB_ITEM, CMD_EXPORT,			"E_xport" },
	//{ CMenuBar::MB_ITEM, CMD_PRINT,			"P_rint" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM, CMD_CLOSE,				"_Close" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem NavigateMenu[] = {
	{ CMenuBar::MB_ITEM, CMD_GOTO_NEXT,			"_Next frame" },
	{ CMenuBar::MB_ITEM, CMD_GOTO_PREV,			"_Previous frame" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM, CMD_GOTO_FIRST,			"_First frame" },
	{ CMenuBar::MB_ITEM, CMD_GOTO_LAST,			"_Last frame" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem ZoomMenu[] = {
	{ CMenuBar::MB_ITEM, CMD_ZOOM_IN,			"Zoom _in" },
	{ CMenuBar::MB_ITEM, CMD_ZOOM_OUT,			"Zoom _out" },
	{ CMenuBar::MB_ITEM, CMD_ZOOM_100,			"Zoom 1:1" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem ViewMenu[] = {
	{ CMenuBar::MB_RADIOBTN, CMD_IMAGE,			"_Image only" },
	{ CMenuBar::MB_RADIOBTN, CMD_CHART,			"_Chart only" },
	{ CMenuBar::MB_RADIOBTN, CMD_IMAGE_CHART,	"I_mage and chart" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_CHECKBTN, CMD_PSEUDOCOLORS,	"_Pseudocolors" },
	{ CMenuBar::MB_CHECKBTN, CMD_BADPIXELS,		"_Bad pixels" },
	{ CMenuBar::MB_CHECKBTN, CMD_ORIGINAL,		"_Original image" },
	{ CMenuBar::MB_CHECKBTN, CMD_GRAYSCALE,		"_Gray scale" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_CHECKBTN, CMD_BADSTARS,		"Bad _stars" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_SUBMENU, 0,					"_Zoom", NULL, ZoomMenu },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenu PreviewMenu[] = {
	{ "_Frame", MENU_FRAME, FrameMenu },
	{ "_Navigate", MENU_NAVIGATE, NavigateMenu },
	{ "_View", MENU_VIEW, ViewMenu },
	{ NULL }
};

//------------------------  COMBO BOX STRINGS   ---------------------------------

static GtkWidget *toolbar_new(GtkOrientation orientation)
{
	GtkWidget *tbox = gtk_toolbar_new();
	gtk_toolbar_set_style(GTK_TOOLBAR(tbox), GTK_TOOLBAR_ICONS);
	gtk_toolbar_set_orientation(GTK_TOOLBAR(tbox), orientation);
	return tbox;
}

static GtkToolItem *toolbar_new_button_from_stock(GtkWidget *toolbar, const gchar *stock_id)
{
	GtkToolItem *tbtn = gtk_tool_button_new_from_stock(stock_id);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tbtn, -1);
	return tbtn;
}

static GtkToolItem *toolbar_new_widget(GtkWidget *toolbar, GtkWidget *widget)
{
	GtkToolItem *item = gtk_tool_item_new();
	gtk_container_add(GTK_CONTAINER(item), widget);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
	return item;
}

static GtkWidget *toolbar_new_entry(GtkWidget *toolbar)
{
	GtkWidget *entry = gtk_entry_new();
	gtk_entry_set_width_chars(GTK_ENTRY(entry), 10);
	gtk_entry_set_alignment(GTK_ENTRY(entry), 0.5);
	gtk_editable_set_editable(GTK_EDITABLE(entry), false);
	toolbar_new_widget(toolbar, entry);
	return entry;
}

static GtkToolItem *toolbar_new_separator(GtkWidget *toolbar)
{
	GtkToolItem *tbtn = gtk_separator_tool_item_new();
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tbtn, -1);
	return tbtn;
}

//-------------------------   PREVIEW DIALOG   --------------------------------

CFrameDlg::CFrameDlg(GtkWindow *parent):m_pFile(NULL), m_InfoMode(INFO_NONE), 
	m_Updating(false), m_DontClose(false), m_FrameID(-1), m_FrameState(0),
	m_LastIndex(-1), m_LastPosX(-1), m_LastPosY(-1), m_PhotReady(false)
{
	char *icon;
	static const GdkGeometry geom = { 320, 200 };
	GtkWidget *tbox, *vbox;

	memset(&m_PhotResult, 0, sizeof(tPhotResult));
	m_DispMode = (tDispMode)CConfig::GetInt("PreviewDlg", "Mode", DISP_FULL);
	m_ShowOrig = CConfig::GetBool("PreviewDlg", "Original");

	// Dialog with buttons
	m_pDlg = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_destroy_with_parent(GTK_WINDOW(m_pDlg), true);
	gtk_window_set_geometry_hints(GTK_WINDOW(m_pDlg), NULL, (GdkGeometry*)&geom, GDK_HINT_MIN_SIZE);
	gtk_window_set_position(GTK_WINDOW(m_pDlg), GTK_WIN_POS_CENTER);
	gtk_window_set_default_size(GTK_WINDOW(m_pDlg), 640, 440);
	g_signal_connect(G_OBJECT(m_pDlg), "destroy", G_CALLBACK(destroy), this);

	// Dialog icon
	icon = get_icon_file("chart");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Window layout
	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(m_pDlg), vbox);

	// Menu bar
	m_Menu.Create(PreviewMenu, false);
	m_Menu.RegisterCallback(MenuClick, this);
	gtk_box_pack_start(GTK_BOX(vbox), m_Menu.Handle(), false, false, 0);

	// Toolbar
	tbox = toolbar_new(GTK_ORIENTATION_HORIZONTAL);
	gtk_box_pack_start(GTK_BOX(vbox), tbox, false, false, 0);
	m_PrevBtn = toolbar_new_button_from_stock(tbox, GTK_STOCK_GO_BACK);
	m_FrameEntry = toolbar_new_entry(tbox);
	m_NextBtn = toolbar_new_button_from_stock(tbox, GTK_STOCK_GO_FORWARD);
	toolbar_new_separator(tbox);
	m_ZoomOut = toolbar_new_button_from_stock(tbox, GTK_STOCK_ZOOM_OUT);
	m_ZoomEntry = toolbar_new_entry(tbox);
	m_ZoomIn = toolbar_new_button_from_stock(tbox, GTK_STOCK_ZOOM_IN);

	// Chart
	m_Chart.SetZoom(CConfig::GetDbl("PreviewDlg", "ZoomFactor", 1.0));
	m_Chart.SetPseudocolors(CConfig::GetBool("PreviewDlg", "Pseudocolors"));
	m_Chart.SetBadPixels(CConfig::GetBool("PreviewDlg", "BadPixels"));
	m_Chart.ShowBadStars(CConfig::GetBool("PreviewDlg", "BadStars", true));
	m_Chart.EnableTracking(true);
	m_Chart.RegisterCallback(ChartCallback, this);
	gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(g_object_ref(m_Chart.Handle())), true, true, 0);

	// Info box
	gtk_box_pack_start(GTK_BOX(vbox), m_InfoBox.Handle(), FALSE, FALSE, 0);
	
	// Scale
	m_ScaleBox.SetChart(&m_Chart);
	gtk_box_pack_start(GTK_BOX(vbox), m_ScaleBox.Handle(), false, true, 0);

	// Status bar
	m_Status.Create();
	gtk_box_pack_start(GTK_BOX(vbox), m_Status.Handle(), FALSE, FALSE, 0);

	// Signals
	g_signal_connect(G_OBJECT(m_PrevBtn), "clicked", G_CALLBACK(button_clicked), this);
	g_signal_connect(G_OBJECT(m_NextBtn), "clicked", G_CALLBACK(button_clicked), this);
	g_signal_connect(G_OBJECT(m_ZoomIn), "clicked", G_CALLBACK(button_clicked), this);
	g_signal_connect(G_OBJECT(m_ZoomOut), "clicked", G_CALLBACK(button_clicked), this);

	// Register callbacks for the model
	GtkTreeModel *pModel = Global.pProject->FileList();
	m_RowDeletedSignal = g_signal_connect(G_OBJECT(pModel), "row-deleted", G_CALLBACK(row_deleted), this);

	gtk_widget_show_all(vbox);
	m_ScaleBox.Show(CConfig::GetBool("PreviewDlg", "GrayScale"));
	m_InfoBox.Show(false);
}

CFrameDlg::~CFrameDlg()
{
	// Disconnect the callback for the model
	if (m_RowDeletedSignal) {
		GtkTreeModel *pModel = Global.pProject->FileList();
		g_signal_handler_disconnect(G_OBJECT(pModel), m_RowDeletedSignal);
	}
	// Free allocated memory
	gtk_tree_row_reference_free(m_pFile);
}

bool CFrameDlg::LoadFrame(GtkTreePath *pPath)
{
	char txt[256];
	GtkTreeIter iter;
	GtkTreeModel *pModel = Global.pProject->FileList();

	m_Updating = true;

	// Get frame data
	gtk_tree_model_get_iter(pModel, &iter, pPath);
	gtk_tree_model_get(pModel, &iter, FRAME_ID, &m_FrameID, FRAME_STATE, &m_FrameState, -1);

	// Window caption
	sprintf(txt, "Frame #%d - preview", m_FrameID);
	gtk_window_set_title(GTK_WINDOW(m_pDlg), txt);

	// Load image
	m_Chart.Clear();
	if (m_ShowOrig || (m_FrameState & CFILE_CONVERSION)==0) {
		char *fpath = Global.pProject->GetSourceFile(pPath);
		m_Chart.LoadImage(fpath);
		g_free(fpath);
	} else {
		char *fpath = Global.pProject->GetImageFile(pPath);
		m_Chart.LoadImage(fpath);
		g_free(fpath);
	}

	// Load chart
	if (m_FrameState & CFILE_PHOTOMETRY) {
		char *fpath = Global.pProject->GetPhotFile(pPath);
		m_Chart.LoadChart(fpath, true);
		g_free(fpath);
	}

	// Set aperture
	m_Chart.SetAperture(Global.pProject->Aperture());

	// Set selection
	if (m_FrameState & CFILE_MATCHING) {
		m_Chart.SetSelection(Global.pProject->Selection());
	}

	// Assign row reference to a actual frame
	gtk_tree_row_reference_free(m_pFile);
	if (pPath)
		m_pFile = gtk_tree_row_reference_new(pModel, pPath);
	else
		m_pFile = NULL;

	m_Updating = false;
	m_PhotReady = false;

	SetDisplayMode(m_DispMode);
	m_Status.Set(NULL);
	m_InfoBox.Show(false);
	UpdateControls();

	return m_pFile!=NULL;
}

void CFrameDlg::ReloadFrame(void)
{
	GtkTreePath *pPath = gtk_tree_row_reference_get_path(m_pFile);
	if (pPath) {
		LoadFrame(pPath);
		gtk_tree_path_free(pPath);
	}
}

void CFrameDlg::UpdateControls(void)
{
	bool ok;
	int *indices, count;
	char txt[256];
	GtkTreePath *pPath, *pPrev, *pNext;
	
	if (m_pFile) {
		pPath = gtk_tree_row_reference_get_path(m_pFile);
		// Frame menu
		m_Menu.EnableMenu(MENU_FRAME, true);
		m_Menu.Enable(CMD_PROPERTIES, true);
		m_Menu.Enable(CMD_REMOVE, true);
		m_Menu.Enable(CMD_EXPORT, true);
		m_Menu.Enable(CMD_PRINT, true);
		// Frame navigation
		m_Menu.EnableMenu(MENU_NAVIGATE, true);
		gtk_widget_set_sensitive(GTK_WIDGET(m_FrameEntry), true);
		pPrev = Global.pProject->GetPreviousFile(pPath);
		gtk_widget_set_sensitive(GTK_WIDGET(m_PrevBtn), pPrev!=NULL);
		m_Menu.Enable(CMD_GOTO_FIRST, pPrev!=NULL);
		m_Menu.Enable(CMD_GOTO_PREV, pPrev!=NULL);
		gtk_tree_path_free(pPrev);
		pNext = Global.pProject->GetNextFile(pPath);
		gtk_widget_set_sensitive(GTK_WIDGET(m_NextBtn), pNext!=NULL);
		m_Menu.Enable(CMD_GOTO_NEXT, pNext!=NULL);
		m_Menu.Enable(CMD_GOTO_LAST, pNext!=NULL);
		gtk_tree_path_free(pNext);
		// View menu
		tDispMode dm = GetDisplayMode();
		m_Menu.Check(CMD_IMAGE, dm==DISP_IMAGE);
		m_Menu.Check(CMD_CHART, dm==DISP_CHART);
		m_Menu.Check(CMD_IMAGE_CHART, dm==DISP_FULL);
		m_Menu.Check(CMD_PSEUDOCOLORS, m_Chart.Pseudocolors());
		m_Menu.Check(CMD_BADPIXELS, m_Chart.BadPixels());
		m_Menu.Check(CMD_GRAYSCALE, m_ScaleBox.Visible());
		m_Menu.Check(CMD_BADSTARS, m_Chart.BadStars());
		m_Menu.EnableMenu(MENU_VIEW, true);
		m_Menu.Enable(CMD_ORIGINAL, (m_FrameState & CFILE_CONVERSION)!=0);
		m_Menu.Enable(CMD_CHART, (m_FrameState & CFILE_PHOTOMETRY)!=0);
		m_Menu.Enable(CMD_IMAGE_CHART, (m_FrameState & CFILE_PHOTOMETRY)!=0);
		m_Menu.Enable(CMD_BADSTARS, (m_FrameState & CFILE_PHOTOMETRY)!=0);
		ok = true;
		gtk_widget_set_sensitive(GTK_WIDGET(m_ZoomOut), ok);
		gtk_widget_set_sensitive(GTK_WIDGET(m_ZoomEntry), ok);
		gtk_widget_set_sensitive(GTK_WIDGET(m_ZoomIn), ok);
		m_Menu.Enable(CMD_ZOOM_IN, ok);
		m_Menu.Enable(CMD_ZOOM_OUT, ok);
		m_Menu.Enable(CMD_ZOOM_100, ok);
		// Frame indicator
		indices = gtk_tree_path_get_indices(pPath);
		count = Global.pProject->GetFileCount();
		sprintf(txt, "%d/%d", indices[0]+1, count);
		gtk_entry_set_text(GTK_ENTRY(m_FrameEntry), txt);
		// Zoom factor
		sprintf(txt, "%.0f %%", 100.0*m_Chart.Zoom());
		gtk_entry_set_text(GTK_ENTRY(m_ZoomEntry), txt);
		gtk_tree_path_free(pPath);
	} else {
		gtk_entry_set_text(GTK_ENTRY(m_FrameEntry), "");
		gtk_entry_set_text(GTK_ENTRY(m_ZoomEntry), "");
		// No frame
		m_Menu.EnableMenu(MENU_FRAME, true);
		m_Menu.Enable(CMD_PROPERTIES, false);
		m_Menu.Enable(CMD_REMOVE, false);
		m_Menu.Enable(CMD_EXPORT, false);
		m_Menu.Enable(CMD_PRINT, false);
		m_Menu.EnableMenu(MENU_NAVIGATE, false);
		m_Menu.EnableMenu(MENU_VIEW, false);
		// Frame section
		gtk_widget_set_sensitive(GTK_WIDGET(m_PrevBtn), false);
		gtk_widget_set_sensitive(GTK_WIDGET(m_NextBtn), false);
		gtk_widget_set_sensitive(GTK_WIDGET(m_FrameEntry), false);
		// Zoom section
		gtk_widget_set_sensitive(GTK_WIDGET(m_ZoomOut), false);
		gtk_widget_set_sensitive(GTK_WIDGET(m_ZoomEntry), false);
		gtk_widget_set_sensitive(GTK_WIDGET(m_ZoomIn), false);
	}
}

void CFrameDlg::MenuClick(int message, int wparam, void *lparam, void *cb_data)
{
	CFrameDlg *pMe = (CFrameDlg*)cb_data;

	switch (message)
	{
	case CMenuBar::CB_ACTIVATE:
		pMe->OnCommand(wparam);
		break;
	}
}

void CFrameDlg::OnCommand(int cmd_id)
{
	switch (cmd_id)
	{
	// Frame menu
	case CMD_CLOSE:
		gtk_widget_destroy(GTK_WIDGET(m_pDlg));
		break;
	case CMD_REMOVE:
		RemoveFromProject();
		break;
	case CMD_PROPERTIES:
		ShowProperties();
		break;

	// Navigate menu
	case CMD_GOTO_FIRST:
		GoToFirstFrame();
		break;
	case CMD_GOTO_LAST:
		GoToLastFrame();
		break;
	case CMD_GOTO_PREV:
		GoToPreviousFrame();
		break;
	case CMD_GOTO_NEXT:
		GoToNextFrame();
		break;

	// View menu
	case CMD_IMAGE:
		SetDisplayMode(DISP_IMAGE);
		break;
	case CMD_CHART:
		SetDisplayMode(DISP_CHART);
		break;
	case CMD_IMAGE_CHART:
		SetDisplayMode(DISP_FULL);
		break;
	case CMD_PSEUDOCOLORS:
		m_Chart.SetPseudocolors(m_Menu.IsChecked(CMD_PSEUDOCOLORS));
		CConfig::SetBool("PreviewDlg", "Pseudocolors", m_Chart.Pseudocolors());
		break;
	case CMD_BADPIXELS:
		m_Chart.SetBadPixels(m_Menu.IsChecked(CMD_BADPIXELS));
		CConfig::SetBool("PreviewDlg", "BadPixels", m_Chart.BadPixels());
		break;
	case CMD_GRAYSCALE:
		m_ScaleBox.Show(m_Menu.IsChecked(CMD_GRAYSCALE));
		CConfig::SetBool("PreviewDlg", "GrayScale", m_ScaleBox.Visible());
		break;
	case CMD_BADSTARS:
		m_Chart.ShowBadStars(m_Menu.IsChecked(CMD_BADSTARS));
		if (!Global.pProject->GetReference(NULL, NULL)) {
			m_Chart.ShowNonMatchedStars(true);
			m_Chart.HighlightNonMatchedStars(false);
		} else {
			m_Chart.ShowNonMatchedStars(m_Chart.BadStars());
			m_Chart.HighlightNonMatchedStars(true);
		}
		CConfig::SetBool("PreviewDlg", "BadStars", m_Chart.BadStars());
		break;
	case CMD_ORIGINAL:
		m_ShowOrig = m_Menu.IsChecked(CMD_ORIGINAL);
		CConfig::SetBool("PreviewDlg", "Original", m_ShowOrig);
		ReloadFrame();
		break;

	// Zoom menu
	case CMD_ZOOM_IN:
		m_Chart.ZoomIn();
		break;
	case CMD_ZOOM_OUT:
		m_Chart.ZoomOut();
		break;
	case CMD_ZOOM_100:
		m_Chart.SetZoom(1.0);
		break;
	}
}

void CFrameDlg::SetDisplayMode(tDispMode mode)
{
	m_DispMode = mode;
	CConfig::SetInt("PreviewDlg", "Mode", m_DispMode);

	tDispMode dm = GetDisplayMode();
	m_Chart.ShowImage(dm!=DISP_CHART);
	m_Chart.ShowChart(dm!=DISP_IMAGE);
	if (!Global.pProject->GetReference(NULL, NULL)) {
		m_Chart.ShowNonMatchedStars(true);
		m_Chart.HighlightNonMatchedStars(false);
	} else {
		m_Chart.ShowNonMatchedStars(m_Chart.BadStars());
		m_Chart.HighlightNonMatchedStars(true);
	}
	m_PhotReady = false;
	m_InfoBox.Show(false);
	UpdateControls();
}

CFrameDlg::tDispMode CFrameDlg::GetDisplayMode() const
{
	if ((m_FrameState & CFILE_PHOTOMETRY)!=0)
		return m_DispMode;
	else
		return DISP_IMAGE;
}

void CFrameDlg::ShowProperties(void)
{
	if (!m_pFile)
		return;

	GtkTreePath *pPath = gtk_tree_row_reference_get_path(m_pFile);
	if (pPath) {
		CFrameInfoDlg dlg(GTK_WINDOW(m_pDlg));
		dlg.Show(pPath);
		gtk_tree_path_free(pPath);
	}
}

void CFrameDlg::button_clicked(GtkWidget *button, CFrameDlg *pDlg)
{
	pDlg->OnButtonClicked(button);
}

void CFrameDlg::OnButtonClicked(GtkWidget *pBtn)
{
	if (pBtn==GTK_WIDGET(m_NextBtn))
		GoToNextFrame();
	else if (pBtn==GTK_WIDGET(m_PrevBtn))
		GoToPreviousFrame();
	else if (pBtn==GTK_WIDGET(m_ZoomIn))
		m_Chart.ZoomIn();
	else if (pBtn==GTK_WIDGET(m_ZoomOut))
		m_Chart.ZoomOut();
}

void CFrameDlg::ChartCallback(int message, int wparam, void* lparam, void* cb_data)
{
	CFrameDlg *pMe = (CFrameDlg*)cb_data;

	switch (message)
	{
	case CChartBase::CB_ZOOM_CHANGE:
		pMe->OnZoomChanged();
		break;
	case CChartBase::CB_MOUSE_MOVE:
		pMe->OnMouseMoved();
		break;
	case CChartBase::CB_LEFT_CLICK:
		pMe->OnLButtonClick(((GdkEventButton*)lparam)->time);
		break;
	}
}


void CFrameDlg::OnZoomChanged(void)
{
	char buf[256];

	sprintf(buf, "%.0f %%", 100.0*m_Chart.Zoom());
	gtk_entry_set_text(GTK_ENTRY(m_ZoomEntry), buf);
}


void CFrameDlg::OnMouseMoved(void)
{
	char	buf[256], aux[128];
	int		x, y, index;
	double	val;
	CChartEx::tStarInfo star;

	if (GetDisplayMode()==DISP_IMAGE) {
		if (m_Chart.GetCursorPos(&x, &y)) {
			if (x!=m_LastPosX || y!=m_LastPosY) {
				m_LastPosX = x;
				m_LastPosY = y;
				val = m_Chart.GetValue(x, y);
				sprintf(buf, "Cursor: X = %d, Y = %d, Val = %.1f ADU", x, y, val);
				m_Status.Set(buf);
			}
		} else {
			if (m_LastPosX!=-1 || m_LastPosY!=-1) {
				m_LastPosX = m_LastPosY = -1;
				m_Status.Set(NULL);
			}
		}
	} else {
		if (m_Chart.GetCursorPos(&x, &y)) 
			index = m_Chart.FindStarByPos(x, y, true);
		else
			index = -1;
		if (index>=0) {
			if (m_LastIndex!=index) {
				m_LastIndex = index;
				m_Chart.SetFocusedStar(index);
				m_Chart.GetStar(index, &star);
				sprintf(buf, "Star #%d: X = %.1f, Y = %.1f", star.id, star.x, star.y);
				if (star.mag_valid && star.err_valid) {
					sprintf(aux, ", Mag = %.4f +- %.4f", star.mag, star.err);
					strcat(buf, aux);
				}
				if (star.mag_valid && !star.err_valid) {
					sprintf(aux, ", Mag = %.4f +- ????", star.mag);
					strcat(buf, aux);
				}
				if (!star.mag_valid && !star.err_valid) {
					sprintf(aux, ", Mag = ???? +- ????");
					strcat(buf, aux);
				}
				if (star.matched) {
					sprintf(aux, ", Ref. = %d", star.xref);
					strcat(buf, aux);
				}
				m_Status.Set(buf);
			}
		} else {
			if (m_LastIndex>=0) {
				m_LastIndex = -1;
				m_Chart.SetFocusedStar(-1);
				m_Status.Set(NULL);
			}
		}
	}
}


//
// Left button click
//
void CFrameDlg::OnLButtonClick(guint32 timestamp)
{
	int x, y, index;
	double aperture;

	if (GetDisplayMode()==DISP_IMAGE) {
		// Quick photometry tool
		if (m_Chart.GetCursorPos(&x, &y)) {
			aperture = GetApertureRadius(Global.pProject->Aperture());
			if (aperture>0) {
				m_PhotReady = m_Chart.Photometry(x, y, aperture, &m_PhotResult);
				UpdateInfoBox();
			}
		} else {
			if (m_PhotReady) {
				m_PhotReady = false;
				UpdateInfoBox();
			}
		}
	} else {
		index = m_Chart.FocusedStar();
		if (index>=0) {
			// Star info tool
			if (m_Chart.SelectedStar()!=index) {
				m_Chart.SelectStar(index);
				UpdateInfoBox();
			}
		} else {
			// Hide infobox
			if (m_Chart.SelectedStar()>=0) {
				m_Chart.SelectStar(-1);
				UpdateInfoBox();
			}
		}
	}
}


void CFrameDlg::GoToFirstFrame(void)
{
	GtkTreePath *pFirst = Global.pProject->GetFirstFile();
	if (pFirst) {
		LoadFrame(pFirst);
		gtk_tree_path_free(pFirst);
	}
}

void CFrameDlg::GoToLastFrame(void)
{
	GtkTreePath *pLast = Global.pProject->GetLastFile();
	if (pLast) {
		LoadFrame(pLast);
		gtk_tree_path_free(pLast);
	}
}

void CFrameDlg::GoToNextFrame(void)
{
	GtkTreePath *pPath, *pNext;

	pPath = gtk_tree_row_reference_get_path(m_pFile);
	pNext = Global.pProject->GetNextFile(pPath);
	if (pNext) {
		LoadFrame(pNext);
		gtk_tree_path_free(pNext);
	}
	gtk_tree_path_free(pPath);
}

void CFrameDlg::GoToPreviousFrame(void)
{
	GtkTreePath *pPath, *pPrev;

	pPath = gtk_tree_row_reference_get_path(m_pFile);
	pPrev = Global.pProject->GetPreviousFile(pPath);
	if (pPrev) {
		LoadFrame(pPrev);
		gtk_tree_path_free(pPrev);
	}
	gtk_tree_path_free(pPath);
}

void CFrameDlg::UpdateInfoBox(void)
{
	int index;
	char buf[256];
	CChartEx::tStarInfo star;

	if (GetDisplayMode()==DISP_IMAGE) {
		// Quick photometry tool
		if (m_PhotReady) {
			if (m_InfoMode != INFO_PHOT) {
				m_InfoBox.Clear();
				m_InfoBox.AddHeading(LABEL_CAPTION, 0, 0);
				m_InfoBox.AddField(LABEL_POSITION, 0, 1, "Position:");
				m_InfoBox.AddField(LABEL_HEIGHT, 0, 2, "Max. value:");
				m_InfoBox.AddField(LABEL_FWHM, 0, 3, "FWHM:");
				m_InfoBox.AddField(LABEL_APERTURE, 1, 0, "Aperture:");
				m_InfoBox.AddField(LABEL_SIGNAL, 1, 1, "Signal:");
				m_InfoBox.AddField(LABEL_SKY, 1, 2, "Sky:");
				m_InfoBox.AddField(LABEL_SNR, 1, 3, "S/N ratio:");
				m_InfoMode = INFO_PHOT;
			}
			m_InfoBox.SetHeading(LABEL_CAPTION, "Quick photometry results:", 0);
			sprintf(buf, "%.1f, %.1f", m_PhotResult.center_x, m_PhotResult.center_y);
			m_InfoBox.SetField(LABEL_POSITION, buf);
			sprintf(buf, "%.1f ADU", m_PhotResult.maxval);
			m_InfoBox.SetField(LABEL_HEIGHT, buf);
			sprintf(buf, "%.1f +- %.1f ADU", m_PhotResult.skyval, m_PhotResult.skysig);
			m_InfoBox.SetField(LABEL_SKY, buf);
			if (m_PhotResult.aperture>0) {
				sprintf(buf, "%.1f pxl", m_PhotResult.aperture);
				m_InfoBox.SetField(LABEL_APERTURE, buf);
			} else {
				m_InfoBox.SetField(LABEL_APERTURE, "------");
			}
			if (m_PhotResult.signal>=0) {
				sprintf(buf, "%.1f ADU", m_PhotResult.signal);
				m_InfoBox.SetField(LABEL_SIGNAL, buf);
			} else {
				m_InfoBox.SetField(LABEL_SIGNAL, "------");
			}
			if (m_PhotResult.fwhm>0.0) {
				sprintf(buf, "%.1f pxl", m_PhotResult.fwhm);
				m_InfoBox.SetField(LABEL_FWHM, buf);
			} else {
				m_InfoBox.SetField(LABEL_FWHM, "------");
			}
			if (m_PhotResult.snr>0.0) {
				sprintf(buf, "%.1f dB", -10.0*log10(m_PhotResult.snr));
				m_InfoBox.SetField(LABEL_SNR, buf);
			} else {
				m_InfoBox.SetField(LABEL_SNR, "------");
			}
			m_InfoBox.Show(true);
		} else {
			m_InfoBox.Show(false);
		}
	} else {
		// Show information about selected star
		index = m_Chart.SelectedStar();
		if (index>=0) {
			if (m_InfoMode != INFO_STAR) {
				m_InfoBox.Clear();
				m_InfoBox.AddHeading(LABEL_CAPTION, 0, 0);
				m_InfoBox.AddField(LABEL_POSITION, 0, 1, "Position");
				m_InfoBox.AddField(LABEL_MAGNITUDE, 0, 2, "Brightness");
				m_InfoBox.AddField(LABEL_REFERENCE, 0, 3, "Reference id");
				m_InfoMode = INFO_STAR;
			}
			m_Chart.GetStar(index, &star);
			// Caption
			switch (star.sel_type)
			{
			case ST_VARIABLE:
				sprintf(buf, "Star #%d (variable star %d)", star.id, star.sel_index);
				m_InfoBox.SetHeading(LABEL_CAPTION, buf, "red");
				break;
			case ST_COMPARISON:
				sprintf(buf, "Star #%d (comparison star %d)", star.id, star.sel_index);
				m_InfoBox.SetHeading(LABEL_CAPTION, buf, "green");
				break;
			case ST_CHECK:
				sprintf(buf, "Star #%d (check star %d)", star.id, star.sel_index);
				m_InfoBox.SetHeading(LABEL_CAPTION, buf, "blue");
				break;
			default:
				sprintf(buf, "Star #%d", star.id);
				m_InfoBox.SetHeading(LABEL_CAPTION, buf, "black");
				break;
			}
			// Position
			sprintf(buf, "%.1f, %.1f", star.x, star.y);
			m_InfoBox.SetField(LABEL_POSITION, buf);
			// Brightness
			if (star.mag_valid) {
				if (star.err_valid) 
					sprintf(buf, "%.4f +- %.4f mag", star.mag, star.err);
				else 
					sprintf(buf, "%.4f +- ???? mag", star.mag);
				m_InfoBox.SetField(LABEL_MAGNITUDE, buf);
			} else {
				m_InfoBox.SetField(LABEL_MAGNITUDE, "------");
			}
			// Reference id
			if (star.matched)
				m_InfoBox.SetField(LABEL_REFERENCE, star.xref);
			else
				m_InfoBox.SetField(LABEL_REFERENCE, "------");
			m_InfoBox.Show(true);
		} else {
			m_InfoBox.Show(false);
		}
	}
}


//
// Show dialog
//
bool CFrameDlg::Show(GtkTreePath *path)
{
	if (LoadFrame(path)) {
		gtk_window_present(GTK_WINDOW(m_pDlg));
		return true;
	}
	return false;
}


//
// Window destroyed
//
void CFrameDlg::destroy(GtkObject *pWnd, CFrameDlg *pDlg)
{
	// Tell the main window to remove the dialog from the list
	if (Global.pMainWnd)
		Global.pMainWnd->FrameClosed(pDlg);
	delete pDlg;
}


//
// Frame deleted
//
void CFrameDlg::row_deleted(GtkTreeModel *tree_model, GtkTreePath *path, CFrameDlg *pDlg)
{
	pDlg->OnRowDeleted(tree_model, path);
}
void CFrameDlg::OnRowDeleted(GtkTreeModel *tree_model, GtkTreePath *path)
{
	if (m_pFile && !m_DontClose) {
		GtkTreePath *pPath = gtk_tree_row_reference_get_path(m_pFile);
		if (!pPath)
			Destroy();
		else 
			UpdateControls();
		gtk_tree_path_free(pPath);
	}
}


//
// Destroy the window
//
void CFrameDlg::Destroy(void)
{
	gtk_widget_destroy(m_pDlg);
}


//
// Remove frame from the list
//
void CFrameDlg::RemoveFromProject(void)
{
	GtkTreePath *pPath, *pNext;
	GtkTreeRowReference *pNextRef;

	if (!m_pFile)
		return;

	if (!ShowConfirmation(GTK_WINDOW(m_pDlg), "Do you want to remove the frame from the current project?"))
		return;

	pPath = gtk_tree_row_reference_get_path(m_pFile);
	// Get reference to a row that will be shown when the current item is deleted
	pNext = Global.pProject->GetNextFile(pPath);
	if (!pNext)
		pNext = Global.pProject->GetPreviousFile(pPath);
	if (pNext)
		pNextRef = gtk_tree_row_reference_new(Global.pProject->FileList(), pNext);
	else
		pNextRef = NULL;
	gtk_tree_path_free(pNext);
	// Delete the frame
	m_DontClose = true;
	Global.pProject->RemoveFile(pPath);
	m_DontClose = false;
	gtk_tree_path_free(pPath);
	// Show the next frame
	if (pNextRef) {
		pPath = gtk_tree_row_reference_get_path(pNextRef);
		LoadFrame(pPath);
		gtk_tree_path_free(pPath);
	} else {
		Destroy();
	}
	gtk_tree_row_reference_free(pNextRef);
}
