/**************************************************************

preview_dlg.h (C-Munipack project)
The preview dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef CMPACK_PREVIEW_DLG_H
#define CMPACK_PREVIEW_DLG_H

#include <gtk/gtk.h>

#include "chart_ex.h"
#include "menubar.h"
#include "statusbar.h"
#include "scale.h"
#include "infobox.h"
#include "qphot.h"

class CFrameDlg
{
public:
	// Constructor
	CFrameDlg(GtkWindow *parent);

	// Destructor
	virtual ~CFrameDlg();

	// Display non-modal dialog
	bool Show(GtkTreePath *path);

	// Destroy the window
	void Destroy();

	// Get row reference to a frame
	GtkTreePath *FramePath(void)
	{ return gtk_tree_row_reference_get_path(m_pFile); }

private:
	enum tDispMode {
		DISP_IMAGE,
		DISP_CHART,
		DISP_FULL
	};

	enum tInfoMode {
		INFO_NONE,
		INFO_STAR,
		INFO_PHOT
	};

	GtkWidget			*m_pDlg, *m_FrameEntry, *m_ZoomEntry;
	GtkToolItem			*m_NextBtn, *m_PrevBtn, *m_ZoomIn, *m_ZoomOut;
	CMenuBar			m_Menu;
	CChartEx			m_Chart;
	CScale				m_ScaleBox;
	CStatusBar			m_Status;
	GtkTreeRowReference *m_pFile;
	CInfoBox			m_InfoBox;
	tDispMode			m_DispMode;
	tInfoMode			m_InfoMode;
	bool				m_ShowOrig, m_Updating, m_DontClose;
	int					m_FrameID, m_FrameState;
	int					m_LastIndex, m_LastPosX, m_LastPosY;
	bool				m_PhotReady;
	tPhotResult			m_PhotResult;
	guint				m_RowDeletedSignal;

	void SetDisplayMode(tDispMode mode);
	tDispMode GetDisplayMode(void) const;
	bool LoadFrame(GtkTreePath *pPath);
	void ReloadFrame(void);
	void UpdateControls(void);
	void ShowProperties(void);
	void GoToFirstFrame(void);
	void GoToLastFrame(void);
	void GoToPreviousFrame(void);
	void GoToNextFrame(void);
	void UpdateApertures(void);
	void UpdateInfoBox(void);
	void RemoveFromProject(void);

	void OnCommand(int cmd_id);
	void OnButtonClicked(GtkWidget *pBtn);
	void OnZoomChanged(void);
	void OnMouseMoved(void);
	void OnLButtonClick(guint32 timestamp);
	void OnRowDeleted(GtkTreeModel *tree_model, GtkTreePath  *path);

	static void destroy(GtkObject *pWnd, CFrameDlg *pDlg);
	static void button_clicked(GtkWidget *pButton, CFrameDlg *pDlg);
	static void row_deleted(GtkTreeModel *tree_model, GtkTreePath  *path, CFrameDlg *pDlg);

	static void MenuClick(int message, int wparam, void* lparam, void* cb_data);
	static void ChartCallback(int message, int wparam, void* lparam, void* cb_data);
};

#endif
