/**************************************************************

helcor_dlg.cpp (C-Munipack project)
Heliocentric correction dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <cmunipack.h>

#include "utils.h"
#include "configuration.h"
#include "helcor_dlg.h"
#include "main_dlg.h"

//-------------------------   JD CONVERTER DIALOG   --------------------------------

bool	CHelCorDlg::ms_NoInput = true;
bool	CHelCorDlg::ms_ValidJD = false;
bool	CHelCorDlg::ms_ValidRA = false;
bool	CHelCorDlg::ms_ValidDec = false;
double	CHelCorDlg::ms_LastRA = 0;
double	CHelCorDlg::ms_LastDec = 0;
double	CHelCorDlg::ms_LastJD = 0;

CHelCorDlg::CHelCorDlg(GtkWindow *pParent)
{
	char buf[256];
	GtkWidget *tbox, *label;

	m_ActMode = (tMode)CConfig::GetInt("HelCor", "Mode");

	// Dialog with buttons
	m_pDlg = gtk_dialog_new_with_buttons("Heliocentric correction", pParent, 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
		GTK_STOCK_REFRESH, GTK_RESPONSE_NONE, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);

	// Dialog icon
	gchar *icon = get_icon_file("helcor");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Dialog layout
	tbox = gtk_table_new(5, 3, FALSE);
	gtk_container_add (GTK_CONTAINER(GTK_DIALOG(m_pDlg)->vbox), tbox);
	gtk_container_set_border_width(GTK_CONTAINER(tbox), 8);
	gtk_table_set_row_spacings(GTK_TABLE(tbox), 4);
	gtk_table_set_row_spacing(GTK_TABLE(tbox), 0, 16);
	gtk_table_set_row_spacing(GTK_TABLE(tbox), 2, 16);
	gtk_table_set_row_spacing(GTK_TABLE(tbox), 3, 16);
	gtk_table_set_col_spacings(GTK_TABLE(tbox), 8);

	// Options
	label = gtk_label_new("Compute");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 0, 1, 0, 1);
	m_Mode = gtk_combo_box_new_text();
	gtk_combo_box_append_text(GTK_COMBO_BOX(m_Mode), "Heliocentric correction");
	gtk_combo_box_append_text(GTK_COMBO_BOX(m_Mode), "Heliocentric julian date");
	gtk_combo_box_append_text(GTK_COMBO_BOX(m_Mode), "Geocentric julian date");
	gtk_combo_box_set_active(GTK_COMBO_BOX(m_Mode), m_ActMode);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Mode, 1, 2, 0, 1);

	// Object's coordinates
	label = gtk_label_new("Object - right ascension");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 0, 1, 1, 2);
	m_RA = gtk_entry_new_with_max_length(32);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_RA, 1, 2, 1, 2);
	label = gtk_label_new("[hh mm ss]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 2, 3, 1, 2);
	if (ms_ValidRA && cmpack_ratostr(ms_LastRA, buf, 256)==0)
		gtk_entry_set_text(GTK_ENTRY(m_RA), buf);
	else
		gtk_entry_set_text(GTK_ENTRY(m_RA), "");
	label = gtk_label_new("- declination");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 0, 1, 2, 3);
	m_Dec = gtk_entry_new_with_max_length(32);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Dec, 1, 2, 2, 3);
	label = gtk_label_new("[\xC2\xB1""ddd mm ss]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 2, 3, 2, 3);
	if (ms_ValidDec && cmpack_dectostr(ms_LastDec, buf, 256)==0)
		gtk_entry_set_text(GTK_ENTRY(m_Dec), buf);
	else
		gtk_entry_set_text(GTK_ENTRY(m_Dec), "");

	// Frame layout
	m_Label1 = gtk_label_new("Julian date");
	gtk_misc_set_alignment(GTK_MISC(m_Label1), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Label1, 0, 1, 3, 4);
	m_JD = gtk_entry_new_with_max_length(32);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_JD, 1, 2, 3, 4);
	label = gtk_label_new("[24xxxxx.xxx]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 2, 3, 3, 4);
	if (ms_ValidJD) {
		sprintf(buf, "%.7f", ms_LastJD);
		gtk_entry_set_text(GTK_ENTRY(m_JD), buf);
	}

	m_Label2 = gtk_label_new("Heliocentric correction");
	gtk_misc_set_alignment(GTK_MISC(m_Label2), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Label2, 0, 1, 4, 5);
	m_Edit2 = gtk_entry_new_with_max_length(32);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Edit2, 1, 2, 4, 5);
	gtk_editable_set_editable(GTK_EDITABLE(m_Edit2), false);

	SetMode(m_ActMode);

	gtk_widget_show_all(tbox);

	// Signals:
	GtkWidget *btn = get_dialog_widget_by_response(m_pDlg, GTK_RESPONSE_NONE);
	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(button_clicked), this);
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);
	g_signal_connect(G_OBJECT(m_Mode), "changed", G_CALLBACK(select_changed), this);
}

CHelCorDlg::~CHelCorDlg()
{
	gtk_widget_destroy(m_pDlg);
}

void CHelCorDlg::response_dialog(GtkDialog *pDlg, gint response_id, CHelCorDlg *pMe)
{
	if (response_id == GTK_RESPONSE_NONE) 
		g_signal_stop_emission_by_name(pDlg, "response");
}

void CHelCorDlg::select_changed(GtkComboBox *pWidget, CHelCorDlg *pMe)
{
	pMe->SetMode((tMode)gtk_combo_box_get_active(pWidget));
}

void CHelCorDlg::button_clicked(GtkWidget *pButton, CHelCorDlg *pMe)
{
	int response_id;
	
	response_id = gtk_dialog_get_response_for_widget(GTK_DIALOG(pMe->m_pDlg), pButton);
	if (response_id == GTK_RESPONSE_NONE) {
		pMe->GetData();
		pMe->SetData();
	}
}

void CHelCorDlg::Execute()
{
	gtk_dialog_run(GTK_DIALOG(m_pDlg));
	CConfig::Flush();
}

void CHelCorDlg::SetMode(tMode mode)
{
	m_ActMode = mode;
	switch (m_ActMode) 
	{
	case MODE_HCORR:
		gtk_label_set_label(GTK_LABEL(m_Label1), "Julian date");
		gtk_label_set_label(GTK_LABEL(m_Label2), "Heliocentric correction");
		break;
	case MODE_HJD:
		gtk_label_set_label(GTK_LABEL(m_Label1), "Geocentric julian date");
		gtk_label_set_label(GTK_LABEL(m_Label2), "Heliocentric julian date");
		break;
	case MODE_GJD:
		gtk_label_set_label(GTK_LABEL(m_Label1), "Heliocentric julian date");
		gtk_label_set_label(GTK_LABEL(m_Label2), "Geocentric julian date");
	}
	CConfig::SetInt("HelCor", "Mode", m_ActMode);

	GetData();
	SetData();
}

void CHelCorDlg::GetData()
{
	double ra, dec;
	const char *text;
	char *buf, *ptr;

	ms_NoInput = true;

	// Read object coordinates
	text = gtk_entry_get_text(GTK_ENTRY(m_RA));
	ms_NoInput = ms_NoInput && (strlen(text)==0);
	if (cmpack_strtora(text, &ra)==0) {
		ms_ValidRA = true;
		ms_LastRA = ra;
	} else
		ms_ValidRA = false;
	text = gtk_entry_get_text(GTK_ENTRY(m_Dec));
	ms_NoInput = ms_NoInput && (strlen(text)==0);
	if (cmpack_strtodec(text, &dec)==0) {
		ms_ValidDec = true;
		ms_LastDec = dec;
	} else
		ms_ValidDec = false;
	// Read julian date
	text = gtk_entry_get_text(GTK_ENTRY(m_JD));
	ms_NoInput = ms_NoInput && (strlen(text)==0);
	if (strlen(text)>0) {
		buf = g_strdup(text);
		ptr = strchr(buf, ',');
		if (ptr)
			*ptr = '.';
		ms_LastJD = atof(buf);
		ms_ValidJD = true;
	} else 
		ms_ValidJD = false;
}

void CHelCorDlg::SetData()
{
	double helcor;
	char buf[256];

	if (ms_NoInput) {
		gtk_entry_set_text(GTK_ENTRY(m_Edit2), "");
		return;
	}
	if (!ms_ValidRA || cmpack_ratostr(ms_LastRA, buf, 256)!=0) {
		gtk_entry_set_text(GTK_ENTRY(m_Edit2), "Invalid right ascension");
		return;
	}
	gtk_entry_set_text(GTK_ENTRY(m_RA), buf);

	if (!ms_ValidDec || cmpack_dectostr(ms_LastDec, buf, 256)!=0) {
		gtk_entry_set_text(GTK_ENTRY(m_Edit2), "Invalid declination");
		return;
	}
	gtk_entry_set_text(GTK_ENTRY(m_Dec), buf);

	if (!ms_ValidJD || ms_LastJD<=0) {
		gtk_entry_set_text(GTK_ENTRY(m_Edit2), "Invalid julian date");
		return;
	}
	sprintf(buf, "%.7f", ms_LastJD);
	gtk_entry_set_text(GTK_ENTRY(m_JD), buf);

	if (ms_ValidRA && ms_ValidDec && ms_ValidJD) {
		helcor = cmpack_helcorr(ms_LastJD, ms_LastRA, ms_LastDec);
		switch (m_ActMode)
		{
		case MODE_HCORR:
			sprintf(buf, "%.7f", helcor);
			break;
		case MODE_HJD:
			sprintf(buf, "%.7f", ms_LastJD + helcor);
			break;
		case MODE_GJD:
			sprintf(buf, "%.7f", ms_LastJD - helcor);
			break;
		default:
			strcpy(buf, "");
		}
		gtk_entry_set_text(GTK_ENTRY(m_Edit2), buf);
	}
}
