/**************************************************************

convert_dlg.cpp (C-Munipack project)
The 'Convert files' dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <cmunipack.h>

#include "project.h"
#include "configuration.h"
#include "makecat_dlg.h"
#include "utils.h"
#include "main.h"

//-------------------------   MAIN WINDOW   --------------------------------

CMakeCatFileDlg::CMakeCatFileDlg(GtkWindow *pParent)
{
	GtkWidget *hbox, *frame, *tbox, *label;

	// Dialog with buttons
	m_pDlg = gtk_dialog_new_with_buttons("Make catalogue file", pParent, 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
		GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);

	// Dialog icon
	gchar *icon = get_icon_file("makecat");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Dialog layout
	hbox = gtk_hbox_new(FALSE, 8);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(m_pDlg)->vbox), hbox);

	// Table layout (left pane
	tbox = gtk_table_new(12, 3, FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), tbox, FALSE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(tbox), 4);
	gtk_table_set_row_spacings(GTK_TABLE(tbox), 4);
	gtk_table_set_col_spacings(GTK_TABLE(tbox), 4);
	gtk_table_set_row_spacing(GTK_TABLE(tbox), 0, 8);
	gtk_table_set_row_spacing(GTK_TABLE(tbox), 3, 8);
	gtk_table_set_row_spacing(GTK_TABLE(tbox), 5, 8);
	gtk_table_set_row_spacing(GTK_TABLE(tbox), 10, 8);

	// Information
	label = gtk_label_new("Please, fill in the following information about the observer object, "
		"the observer and the equipment used. These data will be stored in the catalogue file header. "
		"All fields are optional.");
	gtk_widget_set_size_request(label, 320, -1);
	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 3, 0, 1, GTK_FILL, GTK_FILL, 0, 0);

	// Object
	label = gtk_label_new("Object's designation");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
	m_ObjName = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(m_ObjName), MAX_OBJNAME_LEN);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_ObjName, 1, 3, 1, 2);

	label = gtk_label_new("R. A. (2000)");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
	m_ObjRA = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(m_ObjRA), 64);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_ObjRA, 1, 2, 2, 3);
	label = gtk_label_new("[hh mm ss.s]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 2, 3, 2, 3, GTK_FILL, GTK_FILL, 0, 0);

	label = gtk_label_new("Declination (2000)");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 3, 4, GTK_FILL, GTK_FILL, 0, 0);
	m_ObjDec = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(m_ObjDec), 64);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_ObjDec, 1, 2, 3, 4);
	label = gtk_label_new("[+ddd mm ss]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 2, 3, 3, 4, GTK_FILL, GTK_FILL, 0, 0);

	// Observer
	label = gtk_label_new("Observer's name");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 4, 5, GTK_FILL, GTK_FILL, 0, 0);
	m_ObsName = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(m_ObsName), MAX_OBSNAME_LEN);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_ObsName, 1, 3, 4, 5);

	label = gtk_label_new("Observatory");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 5, 6, GTK_FILL, GTK_FILL, 0, 0);
	m_ObsLoc = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(m_ObsLoc), MAX_LOCATION_LEN);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_ObsLoc, 1, 3, 5, 6);

	// Equipment
	label = gtk_label_new("Telescope");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 6, 7, GTK_FILL, GTK_FILL, 0, 0);
	m_Telescope = gtk_entry_new();
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Telescope, 1, 3, 6, 7);
	label = gtk_label_new("CCD Camera");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 7, 8, GTK_FILL, GTK_FILL, 0, 0);
	m_Camera = gtk_entry_new();
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Camera, 1, 3, 7, 8);
	label = gtk_label_new("Color filter");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 8, 9, GTK_FILL, GTK_FILL, 0, 0);
	m_Filter = gtk_entry_new();
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Filter, 1, 3, 8, 9);
	label = gtk_label_new("Field of view");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 9, 10, GTK_FILL, GTK_FILL, 0, 0);
	m_FOV = gtk_entry_new();
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_FOV, 1, 3, 9, 10);
	label = gtk_label_new("Orientation");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 10, 11, GTK_FILL, GTK_FILL, 0, 0);
	m_Orientation = gtk_entry_new();
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Orientation, 1, 3, 10, 11);

	// Notes
	label = gtk_label_new("Notes");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 11, 12, GTK_FILL, GTK_FILL, 0, 0);
	m_Notes = gtk_entry_new();
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Notes, 1, 3, 11, 12);

	// Catalogue file preview
	frame = gtk_frame_new(NULL);
	gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE, TRUE, 0);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(g_object_ref(m_Chart.Handle())));
	gtk_widget_set_size_request(m_Chart.Handle(), 320, -1);
	m_Chart.EnableTracking(false);
	m_Chart.ShowImage(true);
	m_Chart.ShowChart(false);
	m_Chart.HighlightSelectedStars(true);

	gtk_widget_show_all(hbox);
}

CMakeCatFileDlg::~CMakeCatFileDlg()
{
	gtk_widget_destroy(m_pDlg);
}

void CMakeCatFileDlg::Execute()
{
	int res, refid;
	char *tpath, *ppath, *path, *outfile, *defname;
	char *obsname, *obsloc, *telescope, *camera, *filter;
	char *fov, *orient, *notes;
	const char *cat_file, *tmp_file;
	GtkWindow *pMainWnd = Global.pMainWnd->GetHandle();
	GtkFileFilter *filters[2];

	m_Chart.Clear();

	// Check reference file
	Global.pProject->GetReference(&refid, &cat_file);
	if (refid>0) {
		// Load reference frame
		tpath = Global.pProject->GetImageFile(refid);
		m_Chart.LoadImage(tpath);
		ppath = Global.pProject->GetPhotFile(refid);
		m_Chart.LoadChart(ppath, true);
		g_free(ppath);
		g_free(tpath);
	} else if (cat_file) {
		// Load catalogue file
		tmp_file = Global.pProject->GetTempCatFile();
		m_Chart.LoadChart(tmp_file, true);
		ppath = SetFileExtension(tmp_file, "fts");
		m_Chart.LoadImage(ppath);
		g_free(ppath);
	} else {
		ShowError(pMainWnd, "No reference file.");
		return;
	}

	// Check aperture
	m_Chart.SetAperture(Global.pProject->Aperture());

	// Set selection
	m_Chart.SetSelection(Global.pProject->Selection());

	// Restore last values
	CObjectCoords *obj = Global.pProject->ObjectCoords();
	gtk_entry_set_text(GTK_ENTRY(m_ObjName), obj->Name());
	gtk_entry_set_text(GTK_ENTRY(m_ObjRA), obj->RA());
	gtk_entry_set_text(GTK_ENTRY(m_ObjDec), obj->Dec());

	obsname = CConfig::GetStr("MakeCatDlg", "ObsName");
	gtk_entry_set_text(GTK_ENTRY(m_ObsName), obsname);
	g_free(obsname);
	obsloc = CConfig::GetStr("MakeCatDlg", "ObsLoc");
	gtk_entry_set_text(GTK_ENTRY(m_ObsLoc), obsloc);
	g_free(obsloc);
	telescope = CConfig::GetStr("MakeCatDlg", "Telescope");
	gtk_entry_set_text(GTK_ENTRY(m_Telescope), telescope);
	g_free(telescope);
	camera = CConfig::GetStr("MakeCatDlg", "Camera");
	gtk_entry_set_text(GTK_ENTRY(m_Camera), camera);
	g_free(camera);
	filter = CConfig::GetStr("MakeCatDlg", "Filter");
	gtk_entry_set_text(GTK_ENTRY(m_Filter), filter);
	g_free(filter);
	fov = CConfig::GetStr("MakeCatDlg", "FOV");
	gtk_entry_set_text(GTK_ENTRY(m_FOV), fov);
	g_free(fov);
	orient = CConfig::GetStr("MakeCatDlg", "Orientation");
	gtk_entry_set_text(GTK_ENTRY(m_Orientation), orient);
	g_free(orient);
	notes = CConfig::GetStr("MakeCatDlg", "Notes");
	gtk_entry_set_text(GTK_ENTRY(m_Notes), notes);
	g_free(notes);

	// Edit form
	if (gtk_dialog_run(GTK_DIALOG(m_pDlg))!=GTK_RESPONSE_ACCEPT) {
		return;
	}
	gtk_widget_hide(m_pDlg);

	// Save dialog
	GtkWidget *pSaveDlg = gtk_file_chooser_dialog_new("Save catalogue file",
		pMainWnd, GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
	filters[0] = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filters[0], "*.xml");
	gtk_file_filter_set_name(filters[0], "Catalogue files");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(pSaveDlg), filters[0]);
	filters[1] = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filters[1], "*.*");
	gtk_file_filter_set_name(filters[1], "All files");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(pSaveDlg), filters[1]);
	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(pSaveDlg), filters[0]);
	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(pSaveDlg), true);
	path = CConfig::GetStr("MakeCatDlg", "Folder", NULL);
	if (path) {
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(pSaveDlg), path);
		g_free(path);
	}
	if (strlen(gtk_entry_get_text(GTK_ENTRY(m_ObjName)))>0) {
		defname = MakeFileName(gtk_entry_get_text(GTK_ENTRY(m_ObjName)), "xml");
		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(pSaveDlg), defname);
		g_free(defname);
	} else {
		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(pSaveDlg), "newfile.xml");
	}
	if (gtk_dialog_run(GTK_DIALOG(pSaveDlg))!=GTK_RESPONSE_ACCEPT) {
		gtk_widget_destroy(pSaveDlg);
		return;
	}
	outfile = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(pSaveDlg));
	gtk_widget_destroy(pSaveDlg);

	path = g_path_get_dirname(outfile);
	CConfig::SetStr("MakeCatDlg", "Folder", path);
	g_free(path);

	// Make catalogue file
	res = MakeFile(outfile);
	if (res!=0) {
		char *msg = cmpack_formaterror(res);
		ShowError(pMainWnd, msg);
		cmpack_free(msg);
	}
}

int CMakeCatFileDlg::MakeFile(const char *cat_file)
{
	bool make_img = !CConfig::GetBool("Environment", "NoImageForCatalogFile");
	int res, refid, aperture;
	char *f, *ppath, *tpath, *fts_file;
	const char *file, *objname, *objra, *objdec, *obsname, *obsloc;
	const char *telescope, *camera, *filter, *fov, *orient, *notes;
	CmpackCatFile *dst, *src1;
	CmpackPhtFile *src2;
	gchar *contents;
	gsize length;

	fts_file = SetFileExtension(cat_file, "fts");

	// Delete previous files
	g_unlink(cat_file);
	g_unlink(fts_file);

	dst = cmpack_cat_init();
	Global.pProject->GetReference(&refid, &file);
	if (refid>0) {
		// Make catalogue file from a photometry file
		ppath = Global.pProject->GetPhotFile(refid);
		f = g_locale_from_utf8(ppath, -1, NULL, NULL, NULL);
		res = cmpack_pht_load(&src2, f, CMPACK_LOAD_DEFAULT);
		if (res==0) {
			aperture = Global.pProject->Aperture();
			res = cmpack_cat_make(dst, src2, aperture);
			cmpack_unref(src2);
		}
		g_free(f);
		g_free(ppath);
	} else if (file) {
		// Make copy of another catalogue file
		f = g_locale_from_utf8(file, -1, NULL, NULL, NULL);
		res = cmpack_cat_load(&src1, f, CMPACK_LOAD_DEFAULT);
		if (res==0) {
			cmpack_cat_copy(dst, src1);
			cmpack_unref(src1);
		}
		g_free(f);
	} else {
		res = CMPACK_ERR_CANT_OPEN_SRC;
	}
	if (res==0) {
		// File header
		objname = gtk_entry_get_text(GTK_ENTRY(m_ObjName));
		cmpack_cat_pkys(dst, "object", objname, NULL);
		objra = gtk_entry_get_text(GTK_ENTRY(m_ObjRA));
		cmpack_cat_pkys(dst, "ra2000", objra, NULL);
		objdec = gtk_entry_get_text(GTK_ENTRY(m_ObjDec));
		cmpack_cat_pkys(dst, "dec2000", objdec, NULL);

		obsname = gtk_entry_get_text(GTK_ENTRY(m_ObsName));
		CConfig::SetStr("MakeCatDlg", "ObsName", obsname);
		cmpack_cat_pkys(dst, "observer", obsname, NULL);
		obsloc = gtk_entry_get_text(GTK_ENTRY(m_ObsLoc));
		CConfig::SetStr("MakeCatDlg", "ObsLoc", obsloc);
		cmpack_cat_pkys(dst, "observatory", obsloc, NULL);
		telescope = gtk_entry_get_text(GTK_ENTRY(m_Telescope));
		CConfig::SetStr("MakeCatDlg", "Telescope", telescope);
		cmpack_cat_pkys(dst, "telescope", telescope, NULL);
		camera = gtk_entry_get_text(GTK_ENTRY(m_Camera));
		CConfig::SetStr("MakeCatDlg", "Camera", camera);
		cmpack_cat_pkys(dst, "camera", camera, NULL);
		filter = gtk_entry_get_text(GTK_ENTRY(m_Filter));
		CConfig::SetStr("MakeCatDlg", "Filter", filter);
		cmpack_cat_pkys(dst, "filter", filter, NULL);
		fov = gtk_entry_get_text(GTK_ENTRY(m_FOV));
		CConfig::SetStr("MakeCatDlg", "FOV", fov);
		cmpack_cat_pkys(dst, "fov", fov, NULL);
		orient = gtk_entry_get_text(GTK_ENTRY(m_Orientation));
		CConfig::SetStr("MakeCatDlg", "Orientation", orient);
		cmpack_cat_pkys(dst, "orientation", orient, NULL);
		notes = gtk_entry_get_text(GTK_ENTRY(m_Notes));
		CConfig::SetStr("MakeCatDlg", "Notes", notes);
		cmpack_cat_pkys(dst, "comment", notes, NULL);
		// Selection
		Global.pProject->Selection()->Save(dst);
	}
	if (res==0) {
		// Save catalogue file
		f = g_locale_from_utf8(cat_file, -1, NULL, NULL, NULL);
		res = cmpack_cat_save(dst, f);
		g_free(f);
	}
	cmpack_unref(dst);

	if (res==0 && make_img) {
		// Copy an image file
		if (refid>0) {
			tpath = Global.pProject->GetImageFile(refid);
			if (g_file_get_contents(tpath, &contents, &length, NULL)) {
				g_file_set_contents(fts_file, contents, length, NULL);
				g_free(contents);
			}
			g_free(tpath);
		} else if (file) {
			tpath = SetFileExtension(file, "fts");
			if (g_file_get_contents(tpath, &contents, &length, NULL)) {
				g_file_set_contents(fts_file, contents, length, NULL);
				g_free(contents);
			}
			g_free(tpath);
		}
	}
	g_free(fts_file);

	return res;
}
