/**************************************************************

popup.cpp (C-Munipack project)
Popup menu wrapper
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <glib.h>

#include "popup.h"

struct tPopupMenuItemData
{
	CPopupMenu *menu;
	GtkWidget *item;
	int cmd_id;
	//guint handler_id;
};

// Default constructor
CPopupMenu::CPopupMenu()
{
	m_Data = NULL;
	m_hMenu = NULL;
	m_Count = 0;
}

// Destructor
CPopupMenu::~CPopupMenu()
{
	int i;

	for (i=0; i<m_Count; i++) {
		if (m_Data[i].item) {
			//if (m_Data[i].handler_id>0)
			//	g_signal_handler_disconnect(G_OBJECT(m_Data[i].item), m_Data[i].handler_id);
			g_object_unref(m_Data[i].item);
		}
	}
	g_free(m_Data);
}

// Create and initialize the m_hBar
void CPopupMenu::Create(const tPopupMenuItem *items)
{
	int i, j, count;

	m_hMenu = gtk_menu_new();
	// Compute number of items
	count = 0;
	for (i=0; items[i].type!=MB_END; i++)
		count++;
	if (count>0) {
		m_Data = (tPopupMenuItemData*) g_realloc(m_Data, count*sizeof(tPopupMenuItemData));
		for (j=0; items[j].type!=MB_END; j++) {
			m_Data[j].item = NULL;
			switch (items[j].type)
			{
			case MB_ITEM:
				m_Data[j].menu = this;
				m_Data[j].cmd_id = items[j].cmd_id;
				m_Data[j].item = gtk_menu_item_new_with_mnemonic(items[j].text);
				gtk_menu_shell_append(GTK_MENU_SHELL(m_hMenu), GTK_WIDGET(g_object_ref(m_Data[j].item)));
				g_signal_connect(G_OBJECT(m_Data[j].item), "activate", G_CALLBACK(item_activate), &m_Data[j]);
				gtk_widget_show(m_Data[j].item);
				break;

			case MB_SEPARATOR:
				m_Data[j].item = gtk_separator_menu_item_new();
				gtk_menu_shell_append(GTK_MENU_SHELL(m_hMenu), GTK_WIDGET(g_object_ref(m_Data[j].item)));
				gtk_widget_show(m_Data[j].item);
				break;

			default:
				break;
			}
		}
	}
	m_Count = count;
	gtk_widget_show(m_hMenu);
}

void CPopupMenu::Enable(int cmd, bool enable)
{
	int i;

	for (i=0; i<m_Count; i++) {
		if (m_Data[i].cmd_id == cmd) 
			gtk_widget_set_sensitive(m_Data[i].item, enable);
	}
}

void CPopupMenu::Popup(const GdkEventButton *event)
{
	if (m_hMenu) 
		gtk_menu_popup(GTK_MENU(m_hMenu), NULL, NULL, NULL, NULL, event->button, event->time);
}

void CPopupMenu::item_activate(GtkWidget *widget, gpointer data)
{
	((tPopupMenuItemData*)data)->menu->OnActivate(((tPopupMenuItemData*)data)->cmd_id);
}

void CPopupMenu::OnActivate(int cmd_id)
{
	Callback(CB_ACTIVATE, cmd_id);
}
