/**************************************************************

mfind_dlg.h (C-Munipack project)
The preview dialog for read-all files
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef CMPACK_READALL_DLG_H
#define CMPACK_READALL_DLG_H

#include "file_dlg.h"
#include "info_dlg.h"
#include "graph.h"
#include "graph_data.h"
#include "menubar.h"
#include "scale.h"
#include "statusbar.h"
#include "infobox.h"
#include "helper_classes.h"

//
// Non-modal file viewer
//
class CReadAllDlg:public CFileDlg
{
public:
	// Constructor
	CReadAllDlg(GtkWindow *pParent);

	// Destructor
	virtual ~CReadAllDlg();

protected:
	// Load a file
	virtual bool LoadFile(const char *fpath);

	// Maximize window
	virtual void OnBeforeShow(void);

private:
	CMenuBar		m_Menu;

	// Display modes
	enum tDispMode
	{
		DISP_IMAGE,
		DISP_CHART,
		DISP_FULL
	};

	// Selection mode
	enum tSelectMode
	{
		SELECT_VAR,
		SELECT_COMP
	};
	
	CReadAllInfo	m_Info;
	GtkWidget		*m_Toolbar, *m_Status;
	CGraph			m_MagDevGraph, m_LightCurveGraph;
	CGraphData		m_MagDevData, m_LightCurveData;
	CmpackAllFile	*m_AllFile;
	CmpackTable		*m_MagDevTable, *m_LightCurveTable;
	CChart			m_Chart;
	int				m_VarID, m_CompID, m_LastStarIndex, m_LastPtIndex;
	GSList			*m_FileList;
	GtkWidget		*m_VarBtn, *m_CompBtn, *m_SaveMagDev, *m_SaveCurve, *m_SaveChart;
	tSelectMode		m_SelectMode;
	double			m_MinMag, m_MaxMag, m_MaxDev, m_JDMin, m_JDMax;
	bool			m_Updating;
	GtkWidget		*m_ShowImage, *m_ShowChart, *m_ShowFull;
	tDispMode		m_DispMode;

	static int ExecuteProc(class CProgressDlg *sender, void *user_data);
	static gboolean foreach_all_files(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, GSList **list);
	static void toggled(GtkToggleButton *pBtn, CReadAllDlg *pDlg);
	static void clicked(GtkButton *pBtn, CReadAllDlg *pDlg);
	static void ChartCallback(int message, int wparam, void* lparam, void* cb_data);
	static void GraphCallback(int message, int wparam, void* lparam, void* cb_data);
	static void MenuCallback(int message, int wparam, void* lparam, void* cb_data);

	void UpdateControls(void);
	void SetDisplayMode(tDispMode mode);
	void RebuildData(void);
	int ProcessFiles(class CProgressDlg *sender);
	void OnToggled(GtkToggleButton *pBtn);
	void OnClicked(GtkButton *pBtn);
	void SetMode(tSelectMode mode);
	void SaveMagDev(void);
	void SaveChart(void);
	void SaveCurve(void);
	void OnCommand(int cmd_id);
	void OnChartMouseMove(void);
	void OnChartLButtonClick(const GdkEventButton *event);
	void OnGraphMouseMove(void);
	void OnGraphLButtonClick(const GdkEventButton *event);
	void UpdateSelection(void);
	void UpdateLightCurve(void);
	void ShowProperties(void);
};


//
// File properties dialog
//
class CReadAllInfoDlg:public CInfoDlg
{
public:
	// Constructor
	CReadAllInfoDlg(GtkWindow *pParent);

	// Destructor
	virtual ~CReadAllInfoDlg(void);

	// Display dialog
	void ShowModal(const char *path, const CReadAllInfo *data);

private:
	gchar *m_Path;
	GtkWidget *m_HdrBtn;

	void OnButtonClicked(GtkWidget *pBtn);
	void ShowHeader(void);

	static void button_clicked(GtkWidget *pButton, CReadAllInfoDlg *pDlg);
};

#endif
